@@ -1,5 +1,5 @@
 /*
- * Copyright 2007-2018 NVIDIA Corporation.  All rights reserved.
+ * Copyright 2007-2022 NVIDIA Corporation.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -32,6 +32,8 @@
 #ifndef CUDADEBUGGER_H
 #define CUDADEBUGGER_H
 
+
+
 #include <stdlib.h>
 #include "cuda_stdint.h"
 
@@ -80,14 +82,14 @@ extern "C" {
 
 /*--------------------------------- API Version ------------------------------*/
 
-#define CUDBG_API_VERSION_MAJOR       8 /* Major release version number */
+#define CUDBG_API_VERSION_MAJOR      12 /* Major release version number */
 #define CUDBG_API_VERSION_MINOR       0 /* Minor release version number */
-#define CUDBG_API_VERSION_REVISION  129 /* Revision (build) number */
+#define CUDBG_API_VERSION_REVISION  133 /* Revision (build) number */
 
 /*---------------------------------- Constants -------------------------------*/
 
 #define CUDBG_MAX_DEVICES 32  /* Maximum number of supported devices */
-#define CUDBG_MAX_SMS    128  /* Maximum number of SMs per device */
+#define CUDBG_MAX_SMS    256  /* Maximum number of SMs per device */
 #define CUDBG_MAX_WARPS   64  /* Maximum number of warps per SM */
 #define CUDBG_MAX_LANES   32  /* Maximum number of lanes per warp */
 
@@ -134,6 +136,22 @@ typedef enum {
 #define CUDBG_ENABLE_PREEMPTION_DEBUGGING   cudbgEnablePreemptionDebugging
 #define CUDBG_RESUME_FOR_ATTACH_DETACH      cudbgResumeForAttachDetach
 
+/*
+ * Bitmask of the capabilities supported by the debugger front-end
+ */
+#define CUDBG_DEBUGGER_CAPABILITIES         cudbgDebuggerCapabilities
+
+/*
+ * Can be read to detect whether the external debugger implementation
+ * (libcudadebugger.so) is used or not.
+ */
+#define CUDBG_USE_EXTERNAL_DEBUGGER          cudbgUseExternalDebugger
+
+typedef enum {
+    CUDBG_DEBUGGER_CAPABILITY_NONE                  = 0,
+    CUDBG_DEBUGGER_CAPABILITY_LAZY_FUNCTION_LOADING = (1 << 0)
+} CUDBGCapabilityFlags;
+
 /*---------------- Internal Breakpoint Entries for Error Reporting ------------*/
 
 #define CUDBG_REPORT_DRIVER_API_ERROR                   cudbgReportDriverApiError
@@ -193,10 +211,10 @@ typedef enum {
     CUDBG_ERROR_AMBIGUOUS_MEMORY_ADDRESS    = 0x002b,  /* Address cannot be resolved to a GPU unambiguously */
     CUDBG_ERROR_RECURSIVE_API_CALL          = 0x002c,  /* Debug API entry point called from within a debug API callback */
     CUDBG_ERROR_MISSING_DATA                = 0x002d,  /* The requested data is missing */
-
+    CUDBG_ERROR_NOT_SUPPORTED               = 0x002e,  /* Attempted operation is not supported */
 } CUDBGResult;
 
-static const char *CUDBGResultNames[46] = {
+static const char *CUDBGResultNames[] = {
     "CUDBG_SUCCESS",
     "CUDBG_ERROR_UNKNOWN",
     "CUDBG_ERROR_BUFFER_TOO_SMALL",
@@ -243,6 +261,7 @@ static const char *CUDBGResultNames[46] = {
     "CUDBG_ERROR_AMBIGUOUS_MEMORY_ADDRESS",
     "CUDBG_ERROR_RECURSIVE_API_CALL",
     "CUDBG_ERROR_MISSING_DATA",
+    "CUDBG_ERROR_NOT_SUPPORTED",
 };
 
 static _CUDBG_INLINE const char *cudbgGetErrorString (CUDBGResult error)
@@ -326,6 +345,7 @@ typedef enum {
     CUDBG_EVENT_ATTACH_COMPLETE        = 0x00a,   /* Attach complete. */
     CUDBG_EVENT_DETACH_COMPLETE        = 0x00b,   /* Detach complete. */
     CUDBG_EVENT_ELF_IMAGE_UNLOADED     = 0x00c,   /* ELF image for CUDA kernels(s) no longer available */
+    CUDBG_EVENT_FUNCTIONS_LOADED       = 0x00d,   /* A group of functions/kernels have been loaded */
 } CUDBGEventKind;
 
 /*------------------------------- Kernel Origin ------------------------------*/
@@ -683,6 +703,12 @@ typedef struct {
         struct internalError_st {
             CUDBGResult errorType;
         } internalError;
+        struct functionsLoaded_st {
+            uint32_t  dev;
+            uint32_t  count;
+            uint64_t  context;
+            uint64_t  module;
+        } functionsLoaded;
     } cases;
 } CUDBGEvent;
 #pragma pack(pop)
@@ -711,9 +737,32 @@ typedef struct {
     CUDBGKernelType type;
     uint64_t parentGridId;
     CUDBGKernelOrigin origin;
+} CUDBGGridInfo55;
+
+typedef struct {
+    uint32_t dev;
+    uint64_t gridId64;
+    uint32_t tid;
+    uint64_t context;
+    uint64_t module;
+    uint64_t function;
+    uint64_t functionEntry;
+    CuDim3   gridDim;
+    CuDim3   blockDim;
+    CUDBGKernelType type;
+    uint64_t parentGridId;
+    CUDBGKernelOrigin origin;
+    CuDim3   clusterDim;
 } CUDBGGridInfo;
 #pragma pack(pop)
 
+#pragma pack(push,1)
+typedef struct {
+    uint64_t sectionIndex;
+    uint64_t address;
+} CUDBGLoadedFunctionInfo;
+#pragma pack(pop)
+
 typedef void (*CUDBGNotifyNewEventCallback31)(void *data);
 typedef void (*CUDBGNotifyNewEventCallback40)(CUDBGEventCallbackData40 *data);
 typedef void (*CUDBGNotifyNewEventCallback)(CUDBGEventCallbackData *data);
@@ -739,6 +788,8 @@ typedef enum {
     CUDBG_EXCEPTION_WARP_ILLEGAL_ADDRESS = 14,
     CUDBG_EXCEPTION_LANE_NONMIGRATABLE_ATOMSYS = 15,
     CUDBG_EXCEPTION_LANE_INVALID_ATOMSYS = 16,
+    CUDBG_EXCEPTION_CLUSTER_OUT_OF_RANGE_ADDRESS = 17,
+    CUDBG_EXCEPTION_CLUSTER_BLOCK_NOT_PRESENT = 18,
 } CUDBGException_t;
 
 typedef enum {
@@ -797,6 +848,17 @@ typedef struct {
     uint32_t activeLanes;
     uint32_t errorPCValid;
     CUDBGLaneState lane[32];
+} CUDBGWarpState60;
+
+typedef struct {
+    uint64_t gridId;
+    uint64_t errorPC;
+    CuDim3 blockIdx;
+    uint32_t validLanes;
+    uint32_t activeLanes;
+    uint32_t errorPCValid;
+    CUDBGLaneState lane[32];
+    CuDim3 clusterIdx;
 } CUDBGWarpState;
 #pragma pack(pop)
 
@@ -839,6 +901,7 @@ extern uint32_t CUDBG_ENABLE_LAUNCH_BLOCKING;
 extern uint32_t CUDBG_ENABLE_PREEMPTION_DEBUGGING;
 extern uint32_t CUDBG_RESUME_FOR_ATTACH_DETACH;
 extern uint32_t CUDBG_REPORT_DRIVER_API_ERROR_FLAGS;
+extern uint32_t CUDBG_DEBUGGER_CAPABILITIES;
 
 /* Note this has no effect on virtual GPUs (such as NVIDIA GRID) */
 extern uint32_t CUDBG_ENABLE_INTEGRATED_MEMCHECK;
@@ -963,7 +1026,7 @@ struct CUDBGAPI_st {
     /* 5.5 Extensions */
     CUDBGResult (*getNextSyncEvent55)(CUDBGEvent55 *event);
     CUDBGResult (*getNextAsyncEvent55)(CUDBGEvent55 *event);
-    CUDBGResult (*getGridInfo)(uint32_t dev, uint64_t gridId64, CUDBGGridInfo *gridInfo);
+    CUDBGResult (*getGridInfo55)(uint32_t dev, uint64_t gridId64, CUDBGGridInfo55 *gridInfo);
     CUDBGResult (*readGridId)(uint32_t dev, uint32_t sm, uint32_t wp, uint64_t *gridId64);
     CUDBGResult (*getGridStatus)(uint32_t dev, uint64_t gridId64, CUDBGGridStatus *status);
     CUDBGResult (*setKernelLaunchNotificationMode) (CUDBGKernelLaunchNotifyMode mode);
@@ -976,7 +1039,7 @@ struct CUDBGAPI_st {
     CUDBGResult (*getNextEvent)(CUDBGEventQueueType type, CUDBGEvent  *event);
     CUDBGResult (*getElfImageByHandle)(uint32_t devId, uint64_t handle, CUDBGElfImageType type, void *elfImage, uint64_t size);
     CUDBGResult (*resumeWarpsUntilPC)(uint32_t devId, uint32_t sm, uint64_t warpMask, uint64_t virtPC);
-    CUDBGResult (*readWarpState)(uint32_t devId, uint32_t sm, uint32_t wp, CUDBGWarpState *state);
+    CUDBGResult (*readWarpState60)(uint32_t devId, uint32_t sm, uint32_t wp, CUDBGWarpState60 *state);
     CUDBGResult (*readRegisterRange)(uint32_t devId, uint32_t sm, uint32_t wp, uint32_t ln, uint32_t index, uint32_t registers_size, uint32_t *registers);
     CUDBGResult (*readGenericMemory)(uint32_t dev, uint32_t sm, uint32_t wp, uint32_t ln, uint64_t addr, void *buf, uint32_t sz);
     CUDBGResult (*writeGenericMemory)(uint32_t dev, uint32_t sm, uint32_t wp, uint32_t ln, uint64_t addr, const void *buf, uint32_t sz);
@@ -1006,6 +1069,15 @@ struct CUDBGAPI_st {
     CUDBGResult (*getNumUniformPredicates)(uint32_t dev, uint32_t *numPredicates);
     CUDBGResult (*readUniformPredicates)(uint32_t dev, uint32_t sm, uint32_t wp, uint32_t predicates_size, uint32_t *predicates);
     CUDBGResult (*writeUniformPredicates)(uint32_t dev, uint32_t sm, uint32_t wp, uint32_t predicates_size, const uint32_t *predicates);
+
+    /* 11.8 Extensions */
+    CUDBGResult (*getLoadedFunctionInfo) (uint32_t devId, uint64_t handle, CUDBGLoadedFunctionInfo *info, uint32_t numEntries);
+
+    /* 12.0 Extensions */
+    CUDBGResult (*getGridInfo)(uint32_t dev, uint64_t gridId64, CUDBGGridInfo *gridInfo);
+    CUDBGResult (*getClusterDim)(uint32_t dev, uint64_t gridId64, CuDim3 *clusterDim);
+    CUDBGResult (*readWarpState)(uint32_t dev, uint32_t sm, uint32_t wp, CUDBGWarpState *state);
+    CUDBGResult (*readClusterIdx)(uint32_t dev, uint32_t sm, uint32_t wp, CuDim3 *clusterIdx);
 };
 
 #ifdef __cplusplus
