@@ -1,7 +1,7 @@
 //
 // NVIDIA_COPYRIGHT_BEGIN
 //
-// Copyright (c) 2014-2020, NVIDIA CORPORATION.  All rights reserved.
+// Copyright (c) 2014-2022, NVIDIA CORPORATION.  All rights reserved.
 //
 // NVIDIA CORPORATION and its licensors retain all intellectual property
 // and proprietary rights in and to this software, related documentation
@@ -92,8 +92,37 @@ const char *nvrtcGetErrorString(nvrtcResult result);
 nvrtcResult nvrtcVersion(int *major, int *minor);
 
 
-
+/**
+ * \ingroup query
+ * \brief   nvrtcGetNumSupportedArchs sets the output parameter \p numArchs 
+ *          with the number of architectures supported by NVRTC. This can 
+ *          then be used to pass an array to ::nvrtcGetSupportedArchs to
+ *          get the supported architectures.
+ *
+ * \param   [out] numArchs number of supported architectures.
+ * \return
+ *   - \link #nvrtcResult NVRTC_SUCCESS \endlink
+ *   - \link #nvrtcResult NVRTC_ERROR_INVALID_INPUT \endlink
+ *
+ * see    ::nvrtcGetSupportedArchs
+ */
 nvrtcResult nvrtcGetNumSupportedArchs(int* numArchs);
+
+
+/**
+ * \ingroup query
+ * \brief   nvrtcGetSupportedArchs populates the array passed via the output parameter 
+ *          \p supportedArchs with the architectures supported by NVRTC. The array is
+ *          sorted in the ascending order. The size of the array to be passed can be
+ *          determined using ::nvrtcGetNumSupportedArchs.
+ *
+ * \param   [out] supportedArchs sorted array of supported architectures.
+ * \return
+ *   - \link #nvrtcResult NVRTC_SUCCESS \endlink
+ *   - \link #nvrtcResult NVRTC_ERROR_INVALID_INPUT \endlink
+ *
+ * see    ::nvrtcGetNumSupportedArchs
+ */
 nvrtcResult nvrtcGetSupportedArchs(int* supportedArchs);
 
 
@@ -137,7 +166,8 @@ typedef struct _nvrtcProgram *nvrtcProgram;
  * \param   [in]  includeNames Name of each header by which they can be
  *                             included in the CUDA program source.\n
  *                             \p includeNames can be \c NULL when \p numHeaders
- *                             is 0.
+ *                             is 0. These headers must be included with the exact
+ *                             names specified here.
  * \return
  *   - \link #nvrtcResult NVRTC_SUCCESS \endlink
  *   - \link #nvrtcResult NVRTC_ERROR_OUT_OF_MEMORY \endlink
@@ -195,7 +225,7 @@ nvrtcResult nvrtcCompileProgram(nvrtcProgram prog,
 
 /**
  * \ingroup compilation
- * \brief   nvrtcGetPTXSize sets \p ptxSizeRet with the size of the PTX
+ * \brief   nvrtcGetPTXSize sets the value of \p ptxSizeRet with the size of the PTX
  *          generated by the previous compilation of \p prog (including the
  *          trailing \c NULL).
  *
@@ -231,7 +261,7 @@ nvrtcResult nvrtcGetPTX(nvrtcProgram prog, char *ptx);
 
 /**
  * \ingroup compilation
- * \brief   nvrtcGetCUBINSize sets \p cubinSizeRet with the size of the cubin
+ * \brief   nvrtcGetCUBINSize sets the value of \p cubinSizeRet with the size of the cubin
  *          generated by the previous compilation of \p prog. The value of
  *          cubinSizeRet is set to 0 if the value specified to \c -arch is a
  *          virtual architecture instead of an actual architecture.
@@ -267,6 +297,109 @@ nvrtcResult nvrtcGetCUBINSize(nvrtcProgram prog, size_t *cubinSizeRet);
 nvrtcResult nvrtcGetCUBIN(nvrtcProgram prog, char *cubin);
 
 
+#if defined(_WIN32)
+# define __DEPRECATED__(msg) __declspec(deprecated(msg))
+#elif (defined(__GNUC__) && (__GNUC__ < 4 || (__GNUC__ == 4 && __GNUC_MINOR__ < 5 && !defined(__clang__))))
+# define __DEPRECATED__(msg) __attribute__((deprecated))
+#elif (defined(__GNUC__))
+# define __DEPRECATED__(msg) __attribute__((deprecated(msg)))
+#else
+# define __DEPRECATED__(msg)
+#endif
+
+/**
+ * \ingroup compilation
+ * \brief   
+ * DEPRECATION NOTICE: This function will be removed in a future release. Please use
+ * nvrtcGetLTOIRSize (and nvrtcGetLTOIR) instead.
+ */
+__DEPRECATED__("This function will be removed in a future release. Please use nvrtcGetLTOIRSize instead")
+nvrtcResult nvrtcGetNVVMSize(nvrtcProgram prog, size_t *nvvmSizeRet);
+
+/**
+ * \ingroup compilation
+ * \brief   
+ * DEPRECATION NOTICE: This function will be removed in a future release. Please use
+ * nvrtcGetLTOIR (and nvrtcGetLTOIRSize) instead.
+ */
+__DEPRECATED__("This function will be removed in a future release. Please use nvrtcGetLTOIR instead")
+nvrtcResult nvrtcGetNVVM(nvrtcProgram prog, char *nvvm);
+
+#undef __DEPRECATED__
+
+/**
+ * \ingroup compilation
+ * \brief   nvrtcGetLTOIRSize sets the value of \p LTOIRSizeRet with the size of the LTO IR
+ *          generated by the previous compilation of \p prog. The value of
+ *          LTOIRSizeRet is set to 0 if the program was not compiled with 
+ *          \c -dlto.
+ *
+ * \param   [in]  prog       CUDA Runtime Compilation program.
+ * \param   [out] LTOIRSizeRet Size of the generated LTO IR.
+ * \return
+ *   - \link #nvrtcResult NVRTC_SUCCESS \endlink
+ *   - \link #nvrtcResult NVRTC_ERROR_INVALID_INPUT \endlink
+ *   - \link #nvrtcResult NVRTC_ERROR_INVALID_PROGRAM \endlink
+ *
+ * \see     ::nvrtcGetLTOIR
+ */
+nvrtcResult nvrtcGetLTOIRSize(nvrtcProgram prog, size_t *LTOIRSizeRet);
+
+
+/**
+ * \ingroup compilation
+ * \brief   nvrtcGetLTOIR stores the LTO IR generated by the previous compilation
+ *          of \p prog in the memory pointed by \p LTOIR. No LTO IR is available
+ *          if the program was compiled without \c -dlto.
+ *
+ * \param   [in]  prog CUDA Runtime Compilation program.
+ * \param   [out] LTOIR Compiled result.
+ * \return
+ *   - \link #nvrtcResult NVRTC_SUCCESS \endlink
+ *   - \link #nvrtcResult NVRTC_ERROR_INVALID_INPUT \endlink
+ *   - \link #nvrtcResult NVRTC_ERROR_INVALID_PROGRAM \endlink
+ *
+ * \see     ::nvrtcGetLTOIRSize
+ */
+nvrtcResult nvrtcGetLTOIR(nvrtcProgram prog, char *LTOIR);
+
+
+/**
+ * \ingroup compilation
+ * \brief   nvrtcGetOptiXIRSize sets the value of \p optixirSizeRet with the size of the OptiX IR
+ *          generated by the previous compilation of \p prog. The value of
+ *          nvrtcGetOptiXIRSize is set to 0 if the program was compiled with 
+ *          options incompatible with OptiX IR generation.
+ *
+ * \param   [in]  prog CUDA Runtime Compilation program.
+ * \param   [out] optixirSizeRet Size of the generated LTO IR.
+ * \return
+ *   - \link #nvrtcResult NVRTC_SUCCESS \endlink
+ *   - \link #nvrtcResult NVRTC_ERROR_INVALID_INPUT \endlink
+ *   - \link #nvrtcResult NVRTC_ERROR_INVALID_PROGRAM \endlink
+ *
+ * \see     ::nvrtcGetOptiXIR
+ */
+nvrtcResult nvrtcGetOptiXIRSize(nvrtcProgram prog, size_t *optixirSizeRet);
+
+
+/**
+ * \ingroup compilation
+ * \brief   nvrtcGetOptiXIR stores the OptiX IR generated by the previous compilation
+ *          of \p prog in the memory pointed by \p optixir. No OptiX IR is available
+ *          if the program was compiled with options incompatible with OptiX IR generation.
+ *
+ * \param   [in]  prog CUDA Runtime Compilation program.
+ * \param   [out] Optix IR Compiled result.
+ * \return
+ *   - \link #nvrtcResult NVRTC_SUCCESS \endlink
+ *   - \link #nvrtcResult NVRTC_ERROR_INVALID_INPUT \endlink
+ *   - \link #nvrtcResult NVRTC_ERROR_INVALID_PROGRAM \endlink
+ *
+ * \see     ::nvrtcGetOptiXIRSize
+ */
+nvrtcResult nvrtcGetOptiXIR(nvrtcProgram prog, char *optixir);
+
 /**
  * \ingroup compilation
  * \brief   nvrtcGetProgramLogSize sets \p logSizeRet with the size of the
@@ -381,8 +514,6 @@ nvrtcResult nvrtcGetLoweredName(nvrtcProgram prog,
  *       Specify the name of the class of GPU architectures for which the
  *       input must be compiled.\n
  *       - Valid <c>\<arch\></c>s:
- *         - \c compute_35
- *         - \c compute_37
  *         - \c compute_50
  *         - \c compute_52
  *         - \c compute_53
@@ -393,8 +524,9 @@ nvrtcResult nvrtcGetLoweredName(nvrtcProgram prog,
  *         - \c compute_72
  *         - \c compute_75
  *         - \c compute_80
- *         - \c sm_35
- *         - \c sm_37
+ *         - \c compute_87
+ *         - \c compute_89
+ *         - \c compute_90
  *         - \c sm_50
  *         - \c sm_52
  *         - \c sm_53
@@ -405,6 +537,9 @@ nvrtcResult nvrtcGetLoweredName(nvrtcProgram prog,
  *         - \c sm_72
  *         - \c sm_75
  *         - \c sm_80
+ *         - \c sm_87
+ *         - \c sm_89
+ *         - \c sm_90
  *       - Default: \c compute_52
  *   - Separate compilation / whole-program compilation
  *     - \c --device-c (\c -dc)\n
@@ -421,11 +556,19 @@ nvrtcResult nvrtcGetLoweredName(nvrtcProgram prog,
  *       - Default: \c false
  *   - Debugging support
  *     - \c --device-debug (\c -G)\n
- *       Generate debug information.
+ *       Generate debug information. If --dopt is not specified, 
+ *       then turns off all optimizations.
  *     - \c --generate-line-info (\c -lineinfo)\n
  *       Generate line-number information.
  *   - Code generation
+ *     - \c --dopt on (\c -dopt)\n
+ *     - \c --dopt=on \n
+ *       Enable device code optimization. When specified along with '-G', enables
+ *       limited debug information generation for optimized device code (currently,
+ *       only line number information).
+ *       When '-G' is not specified, '-dopt=on' is implicit.
  *     - \c --ptxas-options \<options\> (\c -Xptxas)\n
+ *     - \c --ptxas-options=\<options\> \n
  *       Specify options directly to ptxas, the PTX optimizing assembler.
  *     - \c --maxrregcount=\<N\> (\c -maxrregcount)\n
  *       Specify the maximum amount of registers that GPU functions can use.
@@ -470,6 +613,19 @@ nvrtcResult nvrtcGetLoweredName(nvrtcProgram prog,
  *       end of compilation.
  *       Note: \c setrlimit() changes the value for the entire process.
  *       - Default: \c true
+ *     - \c --dlink-time-opt (\c -dlto)\n
+ *       Generate intermediate code for later link-time optimization.
+ *       It implies \c -rdc=true. 
+ *       Note: when this option is used the nvrtcGetLTOIR API should be used, 
+ *       as PTX or Cubin will not be generated.
+ *     - \c --gen-opt-lto (\c -gen-opt-lto)\n
+ *       Run the optimizer passes before generating the LTO IR.
+ *     - \c --optix-ir (\c -optix-ir)\n
+ *       Generate OptiX IR. The Optix IR is only intended for consumption by OptiX
+ *       through appropriate APIs. This feature is not supported with 
+ *       link-time-optimization (\c -dlto)\n.
+ *       Note: when this option is used the nvrtcGetOptiX API should be used, 
+ *       as PTX or Cubin will not be generated.
  *   - Preprocessing
  *     - \c --define-macro=\<def\> (\c -D)\n
  *       \c \<def\> can be either \c \<name\> or \c \<name=definitions\>.
@@ -488,9 +644,14 @@ nvrtcResult nvrtcGetLoweredName(nvrtcProgram prog,
  *       headers given to ::nvrtcCreateProgram.
  *     - \c --pre-include=\<header\> (\c -include)\n
  *       Preinclude \c \<header\> during preprocessing.
+ *     - \c --no-source-include (\c -no-source-include)
+ *       The preprocessor by default adds the directory of each input sources
+ *       to the include path. This option disables this feature and only
+ *       considers the path specified explicitly.
  *   - Language Dialect
- *     - \c --std={c++03|c++11|c++14|c++17} (\c -std={c++11|c++14|c++17})\n
- *       Set language dialect to C++03, C++11, C++14 or C++17
+ *     - \c --std={c++03|c++11|c++14|c++17|c++20}
+ *       (\c -std={c++11|c++14|c++17|c++20})\n
+ *       Set language dialect to C++03, C++11, C++14, C++17 or C++20
  *     - \c --builtin-move-forward={true|false} (\c -builtin-move-forward)\n
  *       Provide builtin definitions of \c std::move and \c std::forward,
  *       when C++11 language dialect is selected.
@@ -510,6 +671,9 @@ nvrtcResult nvrtcGetLoweredName(nvrtcProgram prog,
  *       (\c -default-device)\n
  *       Treat entities with no execution space annotation as \c __device__
  *       entities.
+ *     - \c --device-int128 (\c -device-int128)\n
+ *       Allow the \c __int128 type in device code. Also causes the macro \c __CUDACC_RTC_INT128__
+ *       to be defined.
  *     - \c --optimization-info=\<kind\> (\c -opt-info)\n
  *       Provide optimization reports for the specified kind of optimization.
  *       The following kind tags are supported:
@@ -518,7 +682,9 @@ nvrtcResult nvrtcGetLoweredName(nvrtcProgram prog,
  *       Embed used compiler's version info into generated PTX/CUBIN 
  *       - Default: \c false
  *     - \c --display-error-number (\c -err-no)\n
- *       Display diagnostic number for warning messages.
+ *       Display diagnostic number for warning messages. (Default)
+ *     - \c --no-display-error-number (\c -no-err-no)\n
+ *       Disables the display of a diagnostic number for warning messages.
  *     - \c --diag-error=<error-number>,... (\c -diag-error)\n
  *       Emit error for specified diagnostic message number(s). Message numbers can be separated by comma.
  *     - \c --diag-suppress=<error-number>,... (\c -diag-suppress)\n
