@@ -48,6 +48,11 @@ typedef struct cuDeviceGetUuid_params_st {
     CUdevice dev;
 } cuDeviceGetUuid_params;
 
+typedef struct cuDeviceGetUuid_v2_params_st {
+    CUuuid *uuid;
+    CUdevice dev;
+} cuDeviceGetUuid_v2_params;
+
 typedef struct cuDeviceGetLuid_params_st {
     char *luid;
     unsigned int *deviceNodeMask;
@@ -93,6 +98,11 @@ typedef struct cuDeviceGetDefaultMemPool_params_st {
     CUdevice dev;
 } cuDeviceGetDefaultMemPool_params;
 
+typedef struct cuFlushGPUDirectRDMAWrites_params_st {
+    CUflushGPUDirectRDMAWritesTarget target;
+    CUflushGPUDirectRDMAWritesScope scope;
+} cuFlushGPUDirectRDMAWrites_params;
+
 typedef struct cuDeviceGetProperties_params_st {
     CUdevprop *prop;
     CUdevice dev;
@@ -128,12 +138,26 @@ typedef struct cuDevicePrimaryCtxReset_v2_params_st {
     CUdevice dev;
 } cuDevicePrimaryCtxReset_v2_params;
 
+typedef struct cuDeviceGetExecAffinitySupport_params_st {
+    int *pi;
+    CUexecAffinityType type;
+    CUdevice dev;
+} cuDeviceGetExecAffinitySupport_params;
+
 typedef struct cuCtxCreate_v2_params_st {
     CUcontext *pctx;
     unsigned int flags;
     CUdevice dev;
 } cuCtxCreate_v2_params;
 
+typedef struct cuCtxCreate_v3_params_st {
+    CUcontext *pctx;
+    CUexecAffinityParam *paramsArray;
+    int numParams;
+    unsigned int flags;
+    CUdevice dev;
+} cuCtxCreate_v3_params;
+
 typedef struct cuCtxDestroy_v2_params_st {
     CUcontext ctx;
 } cuCtxDestroy_v2_params;
@@ -198,6 +222,11 @@ typedef struct cuCtxGetStreamPriorityRange_params_st {
     int *greatestPriority;
 } cuCtxGetStreamPriorityRange_params;
 
+typedef struct cuCtxGetExecAffinity_params_st {
+    CUexecAffinityParam *pExecAffinity;
+    CUexecAffinityType type;
+} cuCtxGetExecAffinity_params;
+
 typedef struct cuCtxAttach_params_st {
     CUcontext *pctx;
     unsigned int flags;
@@ -247,18 +276,6 @@ typedef struct cuModuleGetGlobal_v2_params_st {
     const char *name;
 } cuModuleGetGlobal_v2_params;
 
-typedef struct cuModuleGetTexRef_params_st {
-    CUtexref *pTexRef;
-    CUmodule hmod;
-    const char *name;
-} cuModuleGetTexRef_params;
-
-typedef struct cuModuleGetSurfRef_params_st {
-    CUsurfref *pSurfRef;
-    CUmodule hmod;
-    const char *name;
-} cuModuleGetSurfRef_params;
-
 typedef struct cuLinkCreate_v2_params_st {
     unsigned int numOptions;
     CUjit_option *options;
@@ -296,6 +313,18 @@ typedef struct cuLinkDestroy_params_st {
     CUlinkState state;
 } cuLinkDestroy_params;
 
+typedef struct cuModuleGetTexRef_params_st {
+    CUtexref *pTexRef;
+    CUmodule hmod;
+    const char *name;
+} cuModuleGetTexRef_params;
+
+typedef struct cuModuleGetSurfRef_params_st {
+    CUsurfref *pSurfRef;
+    CUmodule hmod;
+    const char *name;
+} cuModuleGetSurfRef_params;
+
 typedef struct cuMemGetInfo_v2_params_st {
     size_t *free;
     size_t *total;
@@ -664,6 +693,18 @@ typedef struct cuMipmappedArrayGetSparseProperties_params_st {
     CUmipmappedArray mipmap;
 } cuMipmappedArrayGetSparseProperties_params;
 
+typedef struct cuArrayGetMemoryRequirements_params_st {
+    CUDA_ARRAY_MEMORY_REQUIREMENTS *memoryRequirements;
+    CUarray array;
+    CUdevice device;
+} cuArrayGetMemoryRequirements_params;
+
+typedef struct cuMipmappedArrayGetMemoryRequirements_params_st {
+    CUDA_ARRAY_MEMORY_REQUIREMENTS *memoryRequirements;
+    CUmipmappedArray mipmap;
+    CUdevice device;
+} cuMipmappedArrayGetMemoryRequirements_params;
+
 typedef struct cuArrayGetPlane_params_st {
     CUarray *pPlaneArray;
     CUarray hArray;
@@ -976,10 +1017,26 @@ typedef struct cuStreamIsCapturing_ptsz_params_st {
 
 typedef struct cuStreamGetCaptureInfo_ptsz_params_st {
     CUstream hStream;
-    CUstreamCaptureStatus *captureStatus;
-    cuuint64_t *id;
+    CUstreamCaptureStatus *captureStatus_out;
+    cuuint64_t *id_out;
 } cuStreamGetCaptureInfo_ptsz_params;
 
+typedef struct cuStreamGetCaptureInfo_v2_ptsz_params_st {
+    CUstream hStream;
+    CUstreamCaptureStatus *captureStatus_out;
+    cuuint64_t *id_out;
+    CUgraph *graph_out;
+    const CUgraphNode **dependencies_out;
+    size_t *numDependencies_out;
+} cuStreamGetCaptureInfo_v2_ptsz_params;
+
+typedef struct cuStreamUpdateCaptureDependencies_ptsz_params_st {
+    CUstream hStream;
+    CUgraphNode *dependencies;
+    size_t numDependencies;
+    unsigned int flags;
+} cuStreamUpdateCaptureDependencies_ptsz_params;
+
 typedef struct cuStreamAttachMemAsync_ptsz_params_st {
     CUstream hStream;
     CUdeviceptr dptr;
@@ -1129,6 +1186,41 @@ typedef struct cuStreamBatchMemOp_ptsz_params_st {
     unsigned int flags;
 } cuStreamBatchMemOp_ptsz_params;
 
+typedef struct cuStreamWaitValue32_v2_ptsz_params_st {
+    CUstream stream;
+    CUdeviceptr addr;
+    cuuint32_t value;
+    unsigned int flags;
+} cuStreamWaitValue32_v2_ptsz_params;
+
+typedef struct cuStreamWaitValue64_v2_ptsz_params_st {
+    CUstream stream;
+    CUdeviceptr addr;
+    cuuint64_t value;
+    unsigned int flags;
+} cuStreamWaitValue64_v2_ptsz_params;
+
+typedef struct cuStreamWriteValue32_v2_ptsz_params_st {
+    CUstream stream;
+    CUdeviceptr addr;
+    cuuint32_t value;
+    unsigned int flags;
+} cuStreamWriteValue32_v2_ptsz_params;
+
+typedef struct cuStreamWriteValue64_v2_ptsz_params_st {
+    CUstream stream;
+    CUdeviceptr addr;
+    cuuint64_t value;
+    unsigned int flags;
+} cuStreamWriteValue64_v2_ptsz_params;
+
+typedef struct cuStreamBatchMemOp_v2_ptsz_params_st {
+    CUstream stream;
+    unsigned int count;
+    CUstreamBatchMemOpParams *paramArray;
+    unsigned int flags;
+} cuStreamBatchMemOp_v2_ptsz_params;
+
 typedef struct cuFuncGetAttribute_params_st {
     int *pi;
     CUfunction_attribute attrib;
@@ -1151,6 +1243,11 @@ typedef struct cuFuncSetSharedMemConfig_params_st {
     CUsharedconfig config;
 } cuFuncSetSharedMemConfig_params;
 
+typedef struct cuFuncGetModule_params_st {
+    CUmodule *hmod;
+    CUfunction hfunc;
+} cuFuncGetModule_params;
+
 typedef struct cuLaunchKernel_ptsz_params_st {
     CUfunction f;
     unsigned int gridDimX;
@@ -1420,6 +1517,72 @@ typedef struct cuGraphExternalSemaphoresWaitNodeSetParams_params_st {
     const CUDA_EXT_SEM_WAIT_NODE_PARAMS *nodeParams;
 } cuGraphExternalSemaphoresWaitNodeSetParams_params;
 
+typedef struct cuGraphAddBatchMemOpNode_params_st {
+    CUgraphNode *phGraphNode;
+    CUgraph hGraph;
+    const CUgraphNode *dependencies;
+    size_t numDependencies;
+    const CUDA_BATCH_MEM_OP_NODE_PARAMS *nodeParams;
+} cuGraphAddBatchMemOpNode_params;
+
+typedef struct cuGraphBatchMemOpNodeGetParams_params_st {
+    CUgraphNode hNode;
+    CUDA_BATCH_MEM_OP_NODE_PARAMS *nodeParams_out;
+} cuGraphBatchMemOpNodeGetParams_params;
+
+typedef struct cuGraphBatchMemOpNodeSetParams_params_st {
+    CUgraphNode hNode;
+    const CUDA_BATCH_MEM_OP_NODE_PARAMS *nodeParams;
+} cuGraphBatchMemOpNodeSetParams_params;
+
+typedef struct cuGraphExecBatchMemOpNodeSetParams_params_st {
+    CUgraphExec hGraphExec;
+    CUgraphNode hNode;
+    const CUDA_BATCH_MEM_OP_NODE_PARAMS *nodeParams;
+} cuGraphExecBatchMemOpNodeSetParams_params;
+
+typedef struct cuGraphAddMemAllocNode_params_st {
+    CUgraphNode *phGraphNode;
+    CUgraph hGraph;
+    const CUgraphNode *dependencies;
+    size_t numDependencies;
+    CUDA_MEM_ALLOC_NODE_PARAMS *nodeParams;
+} cuGraphAddMemAllocNode_params;
+
+typedef struct cuGraphMemAllocNodeGetParams_params_st {
+    CUgraphNode hNode;
+    CUDA_MEM_ALLOC_NODE_PARAMS *params_out;
+} cuGraphMemAllocNodeGetParams_params;
+
+typedef struct cuGraphAddMemFreeNode_params_st {
+    CUgraphNode *phGraphNode;
+    CUgraph hGraph;
+    const CUgraphNode *dependencies;
+    size_t numDependencies;
+    CUdeviceptr dptr;
+} cuGraphAddMemFreeNode_params;
+
+typedef struct cuGraphMemFreeNodeGetParams_params_st {
+    CUgraphNode hNode;
+    CUdeviceptr *dptr_out;
+} cuGraphMemFreeNodeGetParams_params;
+
+typedef struct cuDeviceGraphMemTrim_params_st {
+    CUdevice device;
+} cuDeviceGraphMemTrim_params;
+
+typedef struct cuDeviceGetGraphMemAttribute_params_st {
+    CUdevice device;
+    CUgraphMem_attribute attr;
+    void *value;
+} cuDeviceGetGraphMemAttribute_params;
+
+typedef struct cuDeviceSetGraphMemAttribute_params_st {
+    CUdevice device;
+    CUgraphMem_attribute attr;
+    void *value;
+} cuDeviceSetGraphMemAttribute_params;
+
 typedef struct cuGraphClone_params_st {
     CUgraph *phGraphClone;
     CUgraph originalGraph;
@@ -1493,6 +1656,12 @@ typedef struct cuGraphInstantiate_v2_params_st {
     size_t bufferSize;
 } cuGraphInstantiate_v2_params;
 
+typedef struct cuGraphInstantiateWithFlags_params_st {
+    CUgraphExec *phGraphExec;
+    CUgraph hGraph;
+    unsigned long long flags;
+} cuGraphInstantiateWithFlags_params;
+
 typedef struct cuGraphExecKernelNodeSetParams_params_st {
     CUgraphExec hGraphExec;
     CUgraphNode hNode;
@@ -1549,6 +1718,18 @@ typedef struct cuGraphExecExternalSemaphoresWaitNodeSetParams_params_st {
     const CUDA_EXT_SEM_WAIT_NODE_PARAMS *nodeParams;
 } cuGraphExecExternalSemaphoresWaitNodeSetParams_params;
 
+typedef struct cuGraphNodeSetEnabled_params_st {
+    CUgraphExec hGraphExec;
+    CUgraphNode hNode;
+    unsigned int isEnabled;
+} cuGraphNodeSetEnabled_params;
+
+typedef struct cuGraphNodeGetEnabled_params_st {
+    CUgraphExec hGraphExec;
+    CUgraphNode hNode;
+    unsigned int *isEnabled;
+} cuGraphNodeGetEnabled_params;
+
 typedef struct cuGraphUpload_ptsz_params_st {
     CUgraphExec hGraphExec;
     CUstream hStream;
@@ -1591,6 +1772,43 @@ typedef struct cuGraphKernelNodeSetAttribute_params_st {
     const CUkernelNodeAttrValue *value;
 } cuGraphKernelNodeSetAttribute_params;
 
+typedef struct cuGraphDebugDotPrint_params_st {
+    CUgraph hGraph;
+    const char *path;
+    unsigned int flags;
+} cuGraphDebugDotPrint_params;
+
+typedef struct cuUserObjectCreate_params_st {
+    CUuserObject *object_out;
+    void *ptr;
+    CUhostFn destroy;
+    unsigned int initialRefcount;
+    unsigned int flags;
+} cuUserObjectCreate_params;
+
+typedef struct cuUserObjectRetain_params_st {
+    CUuserObject object;
+    unsigned int count;
+} cuUserObjectRetain_params;
+
+typedef struct cuUserObjectRelease_params_st {
+    CUuserObject object;
+    unsigned int count;
+} cuUserObjectRelease_params;
+
+typedef struct cuGraphRetainUserObject_params_st {
+    CUgraph graph;
+    CUuserObject object;
+    unsigned int count;
+    unsigned int flags;
+} cuGraphRetainUserObject_params;
+
+typedef struct cuGraphReleaseUserObject_params_st {
+    CUgraph graph;
+    CUuserObject object;
+    unsigned int count;
+} cuGraphReleaseUserObject_params;
+
 typedef struct cuOccupancyMaxActiveBlocksPerMultiprocessor_params_st {
     int *numBlocks;
     CUfunction func;
@@ -1894,11 +2112,6 @@ typedef struct cuGetExportTable_params_st {
     const CUuuid *pExportTableId;
 } cuGetExportTable_params;
 
-typedef struct cuFuncGetModule_params_st {
-    CUmodule *hmod;
-    CUfunction hfunc;
-} cuFuncGetModule_params;
-
 typedef struct cuMemHostRegister_params_st {
     void *p;
     size_t bytesize;
@@ -2597,6 +2810,41 @@ typedef struct cuStreamBatchMemOp_params_st {
     unsigned int flags;
 } cuStreamBatchMemOp_params;
 
+typedef struct cuStreamWriteValue32_v2_params_st {
+    CUstream stream;
+    CUdeviceptr addr;
+    cuuint32_t value;
+    unsigned int flags;
+} cuStreamWriteValue32_v2_params;
+
+typedef struct cuStreamWaitValue32_v2_params_st {
+    CUstream stream;
+    CUdeviceptr addr;
+    cuuint32_t value;
+    unsigned int flags;
+} cuStreamWaitValue32_v2_params;
+
+typedef struct cuStreamWriteValue64_v2_params_st {
+    CUstream stream;
+    CUdeviceptr addr;
+    cuuint64_t value;
+    unsigned int flags;
+} cuStreamWriteValue64_v2_params;
+
+typedef struct cuStreamWaitValue64_v2_params_st {
+    CUstream stream;
+    CUdeviceptr addr;
+    cuuint64_t value;
+    unsigned int flags;
+} cuStreamWaitValue64_v2_params;
+
+typedef struct cuStreamBatchMemOp_v2_params_st {
+    CUstream stream;
+    unsigned int count;
+    CUstreamBatchMemOpParams *paramArray;
+    unsigned int flags;
+} cuStreamBatchMemOp_v2_params;
+
 typedef struct cuMemPrefetchAsync_params_st {
     CUdeviceptr devPtr;
     size_t count;
@@ -2656,10 +2904,19 @@ typedef struct cuStreamIsCapturing_params_st {
 
 typedef struct cuStreamGetCaptureInfo_params_st {
     CUstream hStream;
-    CUstreamCaptureStatus *captureStatus;
-    cuuint64_t *id;
+    CUstreamCaptureStatus *captureStatus_out;
+    cuuint64_t *id_out;
 } cuStreamGetCaptureInfo_params;
 
+typedef struct cuStreamGetCaptureInfo_v2_params_st {
+    CUstream hStream;
+    CUstreamCaptureStatus *captureStatus_out;
+    cuuint64_t *id_out;
+    CUgraph *graph_out;
+    const CUgraphNode **dependencies_out;
+    size_t *numDependencies_out;
+} cuStreamGetCaptureInfo_v2_params;
+
 typedef struct cuGraphUpload_params_st {
     CUgraphExec hGraph;
     CUstream hStream;
@@ -2724,3 +2981,10 @@ typedef struct cuMemAllocFromPoolAsync_params_st {
     CUmemoryPool pool;
     CUstream hStream;
 } cuMemAllocFromPoolAsync_params;
+
+typedef struct cuStreamUpdateCaptureDependencies_params_st {
+    CUstream hStream;
+    CUgraphNode *dependencies;
+    size_t numDependencies;
+    unsigned int flags;
+} cuStreamUpdateCaptureDependencies_params;
