@@ -661,7 +661,58 @@ typedef enum SANITIZER_CBID_DRIVER_API_enum {
    SANITIZER_CBID_DRIVER_API_cuDeviceGetUuid_v2                                           = 647,
    SANITIZER_CBID_DRIVER_API_cuGraphMemAllocNodeGetParams                                 = 648,
    SANITIZER_CBID_DRIVER_API_cuGraphMemFreeNodeGetParams                                  = 649,
-   SANITIZER_CBID_DRIVER_API_SIZE                                                         = 650,
+   SANITIZER_CBID_DRIVER_API_cuGraphNodeSetEnabled                                        = 650,
+   SANITIZER_CBID_DRIVER_API_cuGraphNodeGetEnabled                                        = 651,
+   SANITIZER_CBID_DRIVER_API_cuLaunchKernelEx                                             = 652,
+   SANITIZER_CBID_DRIVER_API_cuLaunchKernelEx_ptsz                                        = 653,
+   SANITIZER_CBID_DRIVER_API_cuArrayGetMemoryRequirements                                 = 654,
+   SANITIZER_CBID_DRIVER_API_cuMipmappedArrayGetMemoryRequirements                        = 655,
+   SANITIZER_CBID_DRIVER_API_cuGraphInstantiateWithParams                                 = 656,
+   SANITIZER_CBID_DRIVER_API_cuGraphInstantiateWithParams_ptsz                            = 657,
+   SANITIZER_CBID_DRIVER_API_cuGraphExecGetFlags                                          = 658,
+   SANITIZER_CBID_DRIVER_API_cuStreamWaitValue32_v2                                       = 659,
+   SANITIZER_CBID_DRIVER_API_cuStreamWaitValue32_v2_ptsz                                  = 660,
+   SANITIZER_CBID_DRIVER_API_cuStreamWaitValue64_v2                                       = 661,
+   SANITIZER_CBID_DRIVER_API_cuStreamWaitValue64_v2_ptsz                                  = 662,
+   SANITIZER_CBID_DRIVER_API_cuStreamWriteValue32_v2                                      = 663,
+   SANITIZER_CBID_DRIVER_API_cuStreamWriteValue32_v2_ptsz                                 = 664,
+   SANITIZER_CBID_DRIVER_API_cuStreamWriteValue64_v2                                      = 665,
+   SANITIZER_CBID_DRIVER_API_cuStreamWriteValue64_v2_ptsz                                 = 666,
+   SANITIZER_CBID_DRIVER_API_cuStreamBatchMemOp_v2                                        = 667,
+   SANITIZER_CBID_DRIVER_API_cuStreamBatchMemOp_v2_ptsz                                   = 668,
+   SANITIZER_CBID_DRIVER_API_cuGraphAddBatchMemOpNode                                     = 669,
+   SANITIZER_CBID_DRIVER_API_cuGraphBatchMemOpNodeGetParams                               = 670,
+   SANITIZER_CBID_DRIVER_API_cuGraphBatchMemOpNodeSetParams                               = 671,
+   SANITIZER_CBID_DRIVER_API_cuGraphExecBatchMemOpNodeSetParams                           = 672,
+   SANITIZER_CBID_DRIVER_API_cuModuleGetLoadingMode                                       = 673,
+   SANITIZER_CBID_DRIVER_API_cuMemGetHandleForAddressRange                                = 674,
+   SANITIZER_CBID_DRIVER_API_cuOccupancyMaxPotentialClusterSize                           = 675,
+   SANITIZER_CBID_DRIVER_API_cuOccupancyMaxActiveClusters                                 = 676,
+   SANITIZER_CBID_DRIVER_API_cuGetProcAddress_v2                                          = 677,
+   SANITIZER_CBID_DRIVER_API_cuLibraryLoadData                                            = 678,
+   SANITIZER_CBID_DRIVER_API_cuLibraryLoadFromFile                                        = 679,
+   SANITIZER_CBID_DRIVER_API_cuLibraryUnload                                              = 680,
+   SANITIZER_CBID_DRIVER_API_cuLibraryGetKernel                                           = 681,
+   SANITIZER_CBID_DRIVER_API_cuLibraryGetModule                                           = 682,
+   SANITIZER_CBID_DRIVER_API_cuKernelGetFunction                                          = 683,
+   SANITIZER_CBID_DRIVER_API_cuLibraryGetGlobal                                           = 684,
+   SANITIZER_CBID_DRIVER_API_cuLibraryGetManaged                                          = 685,
+   SANITIZER_CBID_DRIVER_API_cuKernelGetAttribute                                         = 686,
+   SANITIZER_CBID_DRIVER_API_cuKernelSetAttribute                                         = 687,
+   SANITIZER_CBID_DRIVER_API_cuKernelSetCacheConfig                                       = 688,
+   SANITIZER_CBID_DRIVER_API_cuGraphAddKernelNode_v2                                      = 689,
+   SANITIZER_CBID_DRIVER_API_cuGraphKernelNodeGetParams_v2                                = 690,
+   SANITIZER_CBID_DRIVER_API_cuGraphKernelNodeSetParams_v2                                = 691,
+   SANITIZER_CBID_DRIVER_API_cuGraphExecKernelNodeSetParams_v2                            = 692,
+   SANITIZER_CBID_DRIVER_API_cuStreamGetId                                                = 693,
+   SANITIZER_CBID_DRIVER_API_cuStreamGetId_ptsz                                           = 694,
+   SANITIZER_CBID_DRIVER_API_cuCtxGetId                                                   = 695,
+   SANITIZER_CBID_DRIVER_API_cuGraphExecUpdate_v2                                         = 696,
+   SANITIZER_CBID_DRIVER_API_cuTensorMapEncodeTiled                                       = 697,
+   SANITIZER_CBID_DRIVER_API_cuTensorMapEncodeIm2col                                      = 698,
+   SANITIZER_CBID_DRIVER_API_cuTensorMapReplaceAddress                                    = 699,
+   SANITIZER_CBID_DRIVER_API_cuLibraryGetUnifiedFunction                                  = 700,
+   SANITIZER_CBID_DRIVER_API_SIZE                                                         = 701,
    SANITIZER_CBID_DRIVER_API_FORCE_INT                                                    = 0x7fffffff,
 } SANITIZER_CBID_DRIVER_API;
 
