@@ -1,5 +1,5 @@
 /*
- * Copyright 2018-2020 NVIDIA Corporation.  All rights reserved.
+ * Copyright 2018-2021 NVIDIA Corporation.  All rights reserved.
  *
  * NOTICE TO LICENSEE:
  *
@@ -139,7 +139,8 @@ SanitizerResult SANITIZERAPI sanitizerAddPatches(const void* image,
  *
  * Error and result codes returned by Sanitizer patches.
  * If a patch returns SANITIZER_PATCH_ERROR, the thread
- * will be exited.
+ * will be exited. On Volta and newer architectures, the
+ * full warp which the thread belongs to will be exited.
  */
 typedef enum {
     /**
@@ -333,6 +334,157 @@ typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackDeviceSideMalloc)(v
  */
 typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackDeviceSideFree)(void* userdata, uint64_t pc, void* ptr);
 
+/**
+ * \brief CUDA Barrier action kind.
+ *
+ * Refer to the CUDA Barrier interface section of the CUDA toolkit documentation for
+ * a more extensive description of these actions.
+ */
+typedef enum {
+
+    /**
+     * Invalid action ID.
+     */
+    SANITIZER_CUDA_BARRIER_INVALID                = 0,
+
+    /**
+     * Barrier initialization.
+     */
+    SANITIZER_CUDA_BARRIER_INIT                   = 1,
+
+    /**
+     * Barrier arrive operation. On Hopper and newer architectures,
+     * barrier data is the count argument to the arrive-on operation.
+     */
+    SANITIZER_CUDA_BARRIER_ARRIVE                 = 2,
+
+    /**
+     * Barrier arrive and drop operation. On Hopper and newer architectures,
+     * barrier data is the count argument to the arrive-on operation.
+     */
+    SANITIZER_CUDA_BARRIER_ARRIVE_DROP            = 3,
+
+    /**
+     * Barrier arrive operation without phase completion.
+     * Barrier data is the count argument to the arrive-on operation.
+     */
+    SANITIZER_CUDA_BARRIER_ARRIVE_NOCOMPLETE      = 4,
+
+    /**
+     * Barrier arrive and drop operation without phase completion.
+     * Barrier data is the count argument to the arrive-on operation.
+     */
+    SANITIZER_CUDA_BARRIER_ARRIVE_DROP_NOCOMPLETE = 5,
+
+    /**
+     * Barrier wait operation.
+     */
+    SANITIZER_CUDA_BARRIER_WAIT                   = 6,
+
+    /**
+     * Barrier invalidation.
+     */
+    SANITIZER_CUDA_BARRIER_INVALIDATE             = 7,
+
+    SANITIZER_CUDA_BARRIER_FORCE_INT              = 0x7fffffff
+} Sanitizer_CudaBarrierInstructionKind;
+
+/**
+ * \brief Function type for a CUDA Barrier action callback.
+ *
+ * \p userdata is a pointer to user data. See \ref sanitizerPatchModule
+ * \p pc is the program counter of the patched instruction
+ * \p barrier Barrier address which can be used as a unique identifier
+ * \p kind Barrier action type. See \ref Sanitizer_CudaBarrierInstructionKind
+ * \p data Barrier data. This is specific to each action type, refer to \ref Sanitizer_CudaBarrierInstructionKind
+ */
+typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackCudaBarrier)(void* userdata, uint64_t pc, void* barrier, uint32_t kind, uint32_t data);
+
+/**
+ * \brief Function type for a global to shared memory asynchronous copy.
+ *
+ * \p userdata is a pointer to user data. See \ref sanitizerPatchModule
+ * \p pc is the program counter of the patched instruction
+ * \p src is the address of the global memory being read.
+ * This can be NULL if src-size is 0.
+ * \p dst is the address of the shared memory being written.
+ * This is an offset within the shared memory window
+ * \p accessSize is the size of the access in bytes. Valid values are 4, 8 and 16.
+ */
+typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackMemcpyAsync)(void* userdata, uint64_t pc, void* src, uint32_t dst, uint32_t accessSize);
+
+/**
+ * \brief Function type for a pipeline commit
+ *
+ * This can be generated by a pipeline::producer_commit (C++ API), a pipeline_commit (C API)
+ * or a cp.async.commit_group (PTX API).
+ *
+ * \p userdata is a pointer to user data. See \ref sanitizerPatchModule
+ * \p pc is the program counter of the patched instruction
+ */
+typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackPipelineCommit)(void* userdata, uint64_t pc);
+
+/**
+ * \brief Function type for a pipeline wait
+ *
+ * This can be generated by a pipeline::consumer_wait (C++ API), a pipeline_wait_prior (C API),
+ * cp.async.wait_group or cp.async.wait_all (PTX API).
+ *
+ * \p userdata is a pointer to user data. See \ref sanitizerPatchModule
+ * \p pc is the program counter of the patched instruction
+ * \p groups is the number of groups the pipeline will wait for. 0 is used to wait for all groups.
+ */
+typedef SanitizerPatchResult (SANITIZERAPI *SanitizerCallbackPipelineWait)(void* userdata, uint64_t pc, uint32_t groups);
+
+/**
+ * \brief Function type for a matrix shared memory access callback
+ *
+ * \p userdata is a pointer to user data. See \ref sanitizerPatchModule
+ * \p pc is the program counter of the patched instruction
+ * \p address is the address of the shared memory being read or written.
+ * This is an offset within the shared memory window
+ * \p accessSize is the size of the access in bytes. Valid value is 16.
+ * \p flags contains information about the type of access. See
+ * Sanitizer_DeviceMemoryFlags to interpret this value.
+ * \p count is the number of matrices accessed.
+ * \p newValue is a pointer to the new value being written if the acces is a
+ * write. If the access is a read or an atomic, the pointer will be NULL.
+ */
+typedef SanitizerPatchResult(SANITIZERAPI* SanitizerCallbackMatrixMemoryAccess)(void* userdata, uint64_t pc, uint32_t address, uint32_t accessSize, uint32_t flags, uint32_t count, const void* pNewValue);
+
+/**
+ * \brief Cache control action
+ */
+typedef enum {
+
+    /**
+     * Invalid action ID.
+     */
+    SANITIZER_CACHE_CONTROL_INVALID           = 0,
+
+    /**
+     * Prefetch to L1.
+     */
+    SANITIZER_CACHE_CONTROL_L1_PREFETCH       = 1,
+
+    /**
+     * Prefetch to L2.
+     */
+    SANITIZER_CACHE_CONTROL_L2_PREFETCH       = 2,
+
+    SANITIZER_CACHE_CONTROL_FORCE_INT         = 0x7fffffff
+} Sanitizer_CacheControlInstructionKind;
+
+/**
+ * \brief Function type for a cache control instruction callback
+ *
+ * \p userdata is a pointer to user data. See \ref sanitizerPatchModule
+ * \p pc is the program counter of the patched instruction
+ * \p address is the address of the memory being controlled
+ * \p kind is the type of cache control. See \ref Sanitizer_CacheControlInstructionKind
+ */
+typedef SanitizerPatchResult(SANITIZERAPI* SanitizerCallbackCacheControl)(void* userdata, uint64_t pc, void* address, Sanitizer_CacheControlInstructionKind kind);
+
 /**
  * \brief Instrumentation.
  *
@@ -412,6 +564,54 @@ typedef enum {
      */
     SANITIZER_INSTRUCTION_DEVICE_SIDE_FREE     = 12,
 
+    /**
+     * CUDA Barrier operation. The type of the callback must be
+     * SanitizerCallbackCudaBarrier.
+     */
+    SANITIZER_INSTRUCTION_CUDA_BARRIER         = 13,
+
+    /**
+     * Global to shared memory asynchronous copy. The type of the
+     * callback must be SanitizerCallbackMemcpyAsync.
+     */
+    SANITIZER_INSTRUCTION_MEMCPY_ASYNC         = 14,
+
+    /**
+     * Pipeline commit. The type of the callback must be
+     * SanitizerCallbackPipelineCommit.
+     */
+    SANITIZER_INSTRUCTION_PIPELINE_COMMIT      = 15,
+
+    /**
+     * Pipeline wait. The type of the callback must be
+     * SanitizerCallbackPipelineWait.
+     */
+    SANITIZER_INSTRUCTION_PIPELINE_WAIT        = 16,
+
+    /**
+     * Remote Shared Memory Access. This can be a store or load operation. The
+     * type of the callback must be SanitizerCallbackMemoryAccess.
+     */
+    SANITIZER_INSTRUCTION_REMOTE_SHARED_MEMORY_ACCESS = 17,
+
+    /**
+     * Device-side aligned malloc. The type of the callback must be
+     * SanitizerCallbackDeviceSideMalloc.
+     */
+    SANITIZER_INSTRUCTION_DEVICE_ALIGNED_MALLOC       = 18,
+
+    /**
+     * Matrix shared memory access. The type of the callback must
+     * be SanitizerCallbackMatrixMemoryAccess.
+     */
+    SANITIZER_INSTRUCTION_MATRIX_MEMORY_ACCESS = 19,
+
+    /**
+     * Cache control instruction. The type of the callback must
+     * be SanitizerCallbackCacheControl.
+     */
+    SANITIZER_INSTRUCTION_CACHE_CONTROL        = 20,
+
     SANITIZER_INSTRUCTION_FORCE_INT            = 0x7fffffff
 } Sanitizer_InstructionId;
 
@@ -558,6 +758,42 @@ SanitizerResult SANITIZERAPI sanitizerGetCallbackPcAndSize(CUcontext ctx,
                                                            uint64_t* pc,
                                                            uint64_t* size);
 
+typedef enum {
+    /**
+     * The function is not loaded.
+     */
+    SANITIZER_FUNCTION_NOT_LOADED           = 0x0,
+
+    /**
+     * The function is being loaded.
+     */
+    SANITIZER_FUNCTION_PARTIALLY_LOADED     = 0x1,
+
+    /**
+     * The function is fully loaded.
+     */
+    SANITIZER_FUNCTION_LOADED               = 0x2,
+
+    SANITIZER_FUNCTION_LOADED_FORCE_INT     = 0x7fffffff
+} Sanitizer_FunctionLoadedStatus;
+
+/**
+ *
+ * \brief Get the loading status of a function. Requires a driver version >=515.
+ *
+ * \param[in] func CUDA function for which the loading status is queried.
+ * \param[out] loadingStatus Loading status returned
+ *
+ * \retval SANITIZER_SUCCESS on success
+ * \retval SANITIZER_ERROR_INVALID_PARAMETER if \p func is NULL or if
+ * loadingStatus is NULL.
+ * \retval SANITIZER_ERROR_NOT_SUPPORTED if the loading status cannot be queried
+ * with this driver version.
+ *
+ */
+SanitizerResult SANITIZERAPI sanitizerGetFunctionLoadedStatus(CUfunction func,
+                                                              Sanitizer_FunctionLoadedStatus* loadingStatus);
+
 
 /** @} */ /* END SANITIZER_PATCHING_API */
 
