@@ -1,5 +1,6 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2021 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB
+ * Copyright (C) 2007-2022 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -789,7 +790,7 @@ cuda_decode_physical_register (uint64_t reg, int32_t *result)
 {
   uint32_t dev_id   = cuda_current_device ();
   uint32_t num_regs = device_get_num_registers (dev_id);
-  reg_t last_regnum = num_regs - 1;
+  ULONGEST last_regnum = num_regs - 1;
 
   if (reg < last_regnum)
     {
