@@ -1,6 +1,6 @@
 /* nto-tdep.c - general QNX Neutrino target functionality.
 
-   Copyright (C) 2003-2020 Free Software Foundation, Inc.
+   Copyright (C) 2003-2022 Free Software Foundation, Inc.
 
    Contributed by QNX Software Systems Ltd.
 
@@ -19,7 +19,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-/* NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2007-2021 NVIDIA Corporation
+/* NVIDIA CUDA Debugger CUDA-GDB
+   Copyright (C) 2007-2022 NVIDIA Corporation
    Modified from the original GDB file referenced above by the CUDA-GDB
    team at NVIDIA <cudatools@nvidia.com>. */
 
@@ -153,17 +154,17 @@ nto_get_siginfo_type (struct gdbarch *gdbarch)
   pid_type = arch_type (gdbarch, TYPE_CODE_TYPEDEF,
 			TYPE_LENGTH (int_type) * TARGET_CHAR_BIT, "pid_t");
   TYPE_TARGET_TYPE (pid_type) = int_type;
-  TYPE_TARGET_STUB (pid_type) = 1;
+  pid_type->set_is_stub (true);
   /* uid_t */
   uid_type = arch_type (gdbarch, TYPE_CODE_TYPEDEF,
 			TYPE_LENGTH (uint_type) * TARGET_CHAR_BIT, "uid_t");
   TYPE_TARGET_TYPE (uid_type) = int_type;
-  TYPE_TARGET_STUB (uid_type) = 1;
+  uid_type->set_is_stub (true);
   /* clock_t */
   clock_type = arch_type (gdbarch, TYPE_CODE_TYPEDEF,
 			  TYPE_LENGTH (uint_type) * TARGET_CHAR_BIT, "clock_t");
   TYPE_TARGET_TYPE (clock_type) = uint_type;
-  TYPE_TARGET_STUB (clock_type) = 1;
+  clock_type->set_is_stub (true);
   /* __data */
   sidata_type = arch_composite_type (gdbarch, NULL, TYPE_CODE_UNION);
   /* __pad */
@@ -702,7 +703,7 @@ nto_sniff_abi_note_section (bfd *abfd, asection *sect, void *obj)
       if (sectsize >= namelen + sizeof_Elf_Nhdr
 	  && namelen == sizeof (QNX_NOTE_NAME)
 	  && 0 == strcmp (name, QNX_NOTE_NAME))
-        *(enum gdb_osabi *) obj = GDB_OSABI_QNXNTO;
+	*(enum gdb_osabi *) obj = GDB_OSABI_QNXNTO;
 
       XDELETEVEC (note);
     }
@@ -720,7 +721,7 @@ nto_elf_osabi_sniffer (bfd *abfd)
   return osabi;
 }
 
-static const char *nto_thread_state_str[] =
+static const char * const nto_thread_state_str[] =
 {
   "DEAD",		/* 0  0x00 */
   "RUNNING",	/* 1  0x01 */
@@ -1078,7 +1079,7 @@ nto_gdbarch_init (struct obstack *obstack)
 /* Read AUXV from initial_stack.  */
 LONGEST
 nto_read_auxv_from_initial_stack (CORE_ADDR initial_stack, gdb_byte *readbuf,
-                                  LONGEST len, size_t sizeof_auxv_t)
+				  LONGEST len, size_t sizeof_auxv_t)
 {
   gdb_byte targ32[4]; /* For 32 bit target values.  */
   gdb_byte targ64[8]; /* For 64 bit target values.  */
@@ -1120,13 +1121,13 @@ nto_read_auxv_from_initial_stack (CORE_ADDR initial_stack, gdb_byte *readbuf,
 
   /* Size of pointer is assumed to be 4 bytes (32 bit arch.) */
   data_ofs += (anint + 2) * ptr_size; /* + 2 comes from argc itself and
-                                                NULL terminating pointer in
-                                                argv.  */
+						NULL terminating pointer in
+						argv.  */
 
   /* Now loop over env table:  */
   anint = 0;
   while (target_read_memory (initial_stack + data_ofs, targ64, ptr_size)
-         == 0)
+	 == 0)
     {
       if (extract_unsigned_integer (targ64, ptr_size, byte_order) == 0)
 	anint = 1; /* Keep looping until non-null entry is found.  */
@@ -1142,18 +1143,18 @@ nto_read_auxv_from_initial_stack (CORE_ADDR initial_stack, gdb_byte *readbuf,
     {
       if (target_read_memory (initial_stack + len_read, buff, sizeof_auxv_t)
 	  == 0)
-        {
+	{
 	  /* Both 32 and 64 bit structures have int as the first field.  */
-          const ULONGEST a_type
+	  const ULONGEST a_type
 	    = extract_unsigned_integer (buff, sizeof (targ32), byte_order);
 
-          if (a_type == AT_NULL)
+	  if (a_type == AT_NULL)
 	    break;
 	  buff += sizeof_auxv_t;
 	  len_read += sizeof_auxv_t;
-        }
+	}
       else
-        break;
+	break;
     }
   return len_read;
 }
@@ -1269,30 +1270,30 @@ void _initialize_nto_tdep ();
 void
 _initialize_nto_tdep ()
 {
-  nto_gdbarch_ops = gdbarch_data_register_pre_init (nto_gdbarch_init); 
-  nto_trace (0) ("%s ()\n", __func__); 
-  add_setshow_zinteger_cmd ("nto-debug", class_maintenance, 
-                            &nto_internal_debugging, _("\ 
-Set QNX NTO internal debugging."), _("\ 
-Show QNX NTO internal debugging."), _("\ 
+  nto_gdbarch_ops = gdbarch_data_register_pre_init (nto_gdbarch_init);
+  nto_trace (0) ("%s ()\n", __func__);
+  add_setshow_zinteger_cmd ("nto-debug", class_maintenance,
+                            &nto_internal_debugging, _("\
+Set QNX NTO internal debugging."), _("\
+Show QNX NTO internal debugging."), _("\
 When non-zero, nto specific debug info is\n\
 displayed. Different information is displayed\n\
-for different positive values."), 
-                            NULL, 
+for different positive values."),
+                            NULL,
                             &show_nto_debug, /* FIXME: i18n: QNX NTO internal 
-                                     debugging is %s.  */ 
-                            &setdebuglist, &showdebuglist); 
-  add_setshow_zinteger_cmd ("nto-stop-on-thread-events", class_support, 
-                            &nto_stop_on_thread_events, _("\ 
-Stop on thread events ."), _("\ 
-Show stop on thread events setting."), _("\ 
-When set to 1, stop on thread created and thread destroyed events.\n"), 
-                            NULL, 
-                            NULL, 
-                            &setlist, &showlist); 
-  add_info ("tidinfo", nto_info_tidinfo_command, "List threads for current process." ); 
-  gdb::observers::solib_loaded.attach (nto_solib_added_listener); 
-  nto_gdbarch_data_handle = 
-    gdbarch_data_register_post_init (init_nto_gdbarch_data); 
+                                     debugging is %s.  */
+                            &setdebuglist, &showdebuglist);
+  add_setshow_zinteger_cmd ("nto-stop-on-thread-events", class_support,
+                            &nto_stop_on_thread_events, _("\
+Stop on thread events ."), _("\
+Show stop on thread events setting."), _("\
+When set to 1, stop on thread created and thread destroyed events.\n"),
+                            NULL,
+                            NULL,
+                            &setlist, &showlist);
+  add_info ("tidinfo", nto_info_tidinfo_command, "List threads for current process." );
+  gdb::observers::solib_loaded.attach (nto_solib_added_listener, "nto-tdep");
+  nto_gdbarch_data_handle =
+    gdbarch_data_register_post_init (init_nto_gdbarch_data);
 }
 #endif
