@@ -1,6 +1,6 @@
 /* Python interface to symbol tables.
 
-   Copyright (C) 2008-2016 Free Software Foundation, Inc.
+   Copyright (C) 2008-2022 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -25,7 +25,7 @@
 #include "objfiles.h"
 #include "block.h"
 
-typedef struct stpy_symtab_object {
+struct symtab_object {
   PyObject_HEAD
   /* The GDB Symbol table structure.  */
   struct symtab *symtab;
@@ -33,9 +33,9 @@ typedef struct stpy_symtab_object {
      a doubly-linked list, rooted in the objfile.  This allows
      invalidation of the underlying struct symtab when the objfile is
      deleted.  */
-  struct stpy_symtab_object *prev;
-  struct stpy_symtab_object *next;
-} symtab_object;
+  symtab_object *prev;
+  symtab_object *next;
+};
 
 extern PyTypeObject symtab_object_type
     CPYCHECKER_TYPE_OBJECT_FOR_TYPEDEF ("symtab_object");
@@ -54,19 +54,19 @@ static const struct objfile_data *stpy_objfile_data_key;
       }							 \
   } while (0)
 
-typedef struct salpy_sal_object {
+struct sal_object {
   PyObject_HEAD
   /* The GDB Symbol table structure.  */
-  symtab_object *symtab;
+  PyObject *symtab;
   /* The GDB Symbol table and line structure.  */
   struct symtab_and_line *sal;
   /* A Symtab and line object is associated with an objfile, so keep
      track with a doubly-linked list, rooted in the objfile.  This
      allows invalidation of the underlying struct symtab_and_line
      when the objfile is deleted.  */
-  struct salpy_sal_object *prev;
-  struct salpy_sal_object *next;
-} sal_object;
+  sal_object *prev;
+  sal_object *next;
+};
 
 extern PyTypeObject sal_object_type
     CPYCHECKER_TYPE_OBJECT_FOR_TYPEDEF ("sal_object");
@@ -108,7 +108,7 @@ stpy_get_filename (PyObject *self, void *closure)
   STPY_REQUIRE_VALID (self, symtab);
   filename = symtab_to_filename_for_display (symtab);
 
-  str_obj = host_string_to_python_string (filename);
+  str_obj = host_string_to_python_string (filename).release ();
   return str_obj;
 }
 
@@ -116,13 +116,10 @@ static PyObject *
 stpy_get_objfile (PyObject *self, void *closure)
 {
   struct symtab *symtab = NULL;
-  PyObject *result;
 
   STPY_REQUIRE_VALID (self, symtab);
 
-  result = objfile_to_objfile_object (SYMTAB_OBJFILE (symtab));
-  Py_XINCREF (result);
-  return result;
+  return objfile_to_objfile_object (symtab->objfile ()).release ();
 }
 
 /* Getter function for symtab.producer.  */
@@ -134,12 +131,12 @@ stpy_get_producer (PyObject *self, void *closure)
   struct compunit_symtab *cust;
 
   STPY_REQUIRE_VALID (self, symtab);
-  cust = SYMTAB_COMPUNIT (symtab);
-  if (COMPUNIT_PRODUCER (cust) != NULL)
+  cust = symtab->compunit ();
+  if (cust->producer () != nullptr)
     {
-      const char *producer = COMPUNIT_PRODUCER (cust);
+      const char *producer = cust->producer ();
 
-      return host_string_to_python_string (producer);
+      return host_string_to_python_string (producer).release ();
     }
 
   GDB_PY_RETURN_NONE;
@@ -155,7 +152,7 @@ stpy_fullname (PyObject *self, PyObject *args)
 
   fullname = symtab_to_fullname (symtab);
 
-  return host_string_to_python_string (fullname);
+  return host_string_to_python_string (fullname).release ();
 }
 
 /* Implementation of gdb.Symtab.is_valid (self) -> Boolean.
@@ -179,14 +176,14 @@ static PyObject *
 stpy_global_block (PyObject *self, PyObject *args)
 {
   struct symtab *symtab = NULL;
-  struct block *block = NULL;
+  const struct block *block = NULL;
   const struct blockvector *blockvector;
 
   STPY_REQUIRE_VALID (self, symtab);
 
-  blockvector = SYMTAB_BLOCKVECTOR (symtab);
+  blockvector = symtab->blockvector ();
   block = BLOCKVECTOR_BLOCK (blockvector, GLOBAL_BLOCK);
-  return block_to_block_object (block, SYMTAB_OBJFILE (symtab));
+  return block_to_block_object (block, symtab->objfile ());
 }
 
 /* Return the STATIC_BLOCK of the underlying symtab.  */
@@ -195,14 +192,14 @@ static PyObject *
 stpy_static_block (PyObject *self, PyObject *args)
 {
   struct symtab *symtab = NULL;
-  struct block *block = NULL;
+  const struct block *block = NULL;
   const struct blockvector *blockvector;
 
   STPY_REQUIRE_VALID (self, symtab);
 
-  blockvector = SYMTAB_BLOCKVECTOR (symtab);
+  blockvector = symtab->blockvector ();
   block = BLOCKVECTOR_BLOCK (blockvector, STATIC_BLOCK);
-  return block_to_block_object (block, SYMTAB_OBJFILE (symtab));
+  return block_to_block_object (block, symtab->objfile ());
 }
 
 /* Implementation of gdb.Symtab.linetable (self) -> gdb.LineTable.
@@ -222,25 +219,24 @@ stpy_get_linetable (PyObject *self, PyObject *args)
 static PyObject *
 salpy_str (PyObject *self)
 {
-  char *s;
   const char *filename;
   sal_object *sal_obj;
-  PyObject *result;
   struct symtab_and_line *sal = NULL;
 
   SALPY_REQUIRE_VALID (self, sal);
 
   sal_obj = (sal_object *) self;
-  filename = (sal_obj->symtab == (symtab_object *) gdbpy_None)
-    ? "<unknown>" : symtab_to_filename_for_display (sal_obj->symtab->symtab);
 
-  s = xstrprintf ("symbol and line for %s, line %d", filename,
-		  sal->line);
+  if (sal_obj->symtab == gdbpy_None)
+    filename = "<unknown>";
+  else
+    {
+      symtab *symtab = symtab_object_to_symtab (sal_obj->symtab);
+      filename = symtab_to_filename_for_display (symtab);
+    }
 
-  result = PyString_FromString (s);
-  xfree (s);
-
-  return result;
+  return gdbpy_StringFromFormat ("symbol and line for %s, line %d", filename,
+			      sal->line);
 }
 
 static void
@@ -252,12 +248,13 @@ stpy_dealloc (PyObject *obj)
     symtab->prev->next = symtab->next;
   else if (symtab->symtab)
     {
-      set_objfile_data (SYMTAB_OBJFILE (symtab->symtab),
+      set_objfile_data (symtab->symtab->objfile (),
 			stpy_objfile_data_key, symtab->next);
     }
   if (symtab->next)
     symtab->next->prev = symtab->prev;
   symtab->symtab = NULL;
+  Py_TYPE (obj)->tp_free (obj);
 }
 
 
@@ -268,7 +265,7 @@ salpy_get_pc (PyObject *self, void *closure)
 
   SALPY_REQUIRE_VALID (self, sal);
 
-  return gdb_py_long_from_ulongest (sal->pc);
+  return gdb_py_object_from_ulongest (sal->pc).release ();
 }
 
 /* Implementation of the get method for the 'last' attribute of
@@ -282,7 +279,7 @@ salpy_get_last (PyObject *self, void *closure)
   SALPY_REQUIRE_VALID (self, sal);
 
   if (sal->end > 0)
-    return gdb_py_long_from_ulongest (sal->end - 1);
+    return gdb_py_object_from_ulongest (sal->end - 1).release ();
   else
     GDB_PY_RETURN_NONE;
 }
@@ -294,7 +291,7 @@ salpy_get_line (PyObject *self, void *closure)
 
   SALPY_REQUIRE_VALID (self, sal);
 
-  return PyInt_FromLong (sal->line);
+  return gdb_py_object_from_longest (sal->line).release ();
 }
 
 static PyObject *
@@ -332,9 +329,10 @@ salpy_dealloc (PyObject *self)
 
   if (self_sal->prev)
     self_sal->prev->next = self_sal->next;
-  else if (self_sal->symtab != (symtab_object * ) gdbpy_None)
-    set_objfile_data (SYMTAB_OBJFILE (self_sal->symtab->symtab),
-		      salpy_objfile_data_key, self_sal->next);
+  else if (self_sal->symtab != gdbpy_None)
+    set_objfile_data
+      (symtab_object_to_symtab (self_sal->symtab)->objfile (),
+       salpy_objfile_data_key, self_sal->next);
 
   if (self_sal->next)
     self_sal->next->prev = self_sal->prev;
@@ -352,11 +350,11 @@ salpy_dealloc (PyObject *self)
 static int CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION
 set_sal (sal_object *sal_obj, struct symtab_and_line sal)
 {
-  symtab_object *symtab_obj;
+  PyObject *symtab_obj;
 
   if (sal.symtab)
     {
-      symtab_obj = (symtab_object *) symtab_to_symtab_object  (sal.symtab);
+      symtab_obj = symtab_to_symtab_object  (sal.symtab);
       /* If a symtab existed in the sal, but it cannot be duplicated,
 	 we exit.  */
       if (symtab_obj == NULL)
@@ -364,7 +362,7 @@ set_sal (sal_object *sal_obj, struct symtab_and_line sal)
     }
   else
     {
-      symtab_obj = (symtab_object *) gdbpy_None;
+      symtab_obj = gdbpy_None;
       Py_INCREF (gdbpy_None);
     }
 
@@ -376,16 +374,17 @@ set_sal (sal_object *sal_obj, struct symtab_and_line sal)
 
   /* If the SAL does not have a symtab, we do not add it to the
      objfile cleanup observer linked list.  */
-  if (sal_obj->symtab != (symtab_object *)gdbpy_None)
+  if (sal_obj->symtab != gdbpy_None)
     {
+      symtab *symtab = symtab_object_to_symtab (sal_obj->symtab);
+
       sal_obj->next
-	= ((struct salpy_sal_object *)
-	   objfile_data (SYMTAB_OBJFILE (sal_obj->symtab->symtab),
-			 salpy_objfile_data_key));
+	= ((sal_object *) objfile_data (symtab->objfile (),
+					salpy_objfile_data_key));
       if (sal_obj->next)
 	sal_obj->next->prev = sal_obj;
 
-      set_objfile_data (SYMTAB_OBJFILE (sal_obj->symtab->symtab),
+      set_objfile_data (symtab->objfile (),
 			salpy_objfile_data_key, sal_obj);
     }
   else
@@ -407,11 +406,11 @@ set_symtab (symtab_object *obj, struct symtab *symtab)
   if (symtab)
     {
       obj->next
-	= ((struct stpy_symtab_object *)
-	   objfile_data (SYMTAB_OBJFILE (symtab), stpy_objfile_data_key));
+	= ((symtab_object *)
+	   objfile_data (symtab->objfile (), stpy_objfile_data_key));
       if (obj->next)
 	obj->next->prev = obj;
-      set_objfile_data (SYMTAB_OBJFILE (symtab), stpy_objfile_data_key, obj);
+      set_objfile_data (symtab->objfile (), stpy_objfile_data_key, obj);
     }
   else
     obj->next = NULL;
@@ -436,19 +435,14 @@ symtab_to_symtab_object (struct symtab *symtab)
 PyObject *
 symtab_and_line_to_sal_object (struct symtab_and_line sal)
 {
-  sal_object *sal_obj;
-
-  sal_obj = PyObject_New (sal_object, &sal_object_type);
-  if (sal_obj)
+  gdbpy_ref<sal_object> sal_obj (PyObject_New (sal_object, &sal_object_type));
+  if (sal_obj != NULL)
     {
-      if (set_sal (sal_obj, sal) < 0)
-	{
-	  Py_DECREF (sal_obj);
-	  return NULL;
-	}
+      if (set_sal (sal_obj.get (), sal) < 0)
+	return NULL;
     }
 
-  return (PyObject *) sal_obj;
+  return (PyObject *) sal_obj.release ();
 }
 
 /* Return struct symtab_and_line reference that is wrapped by this
@@ -505,8 +499,8 @@ del_objfile_sal (struct objfile *objfile, void *datum)
     {
       sal_object *next = obj->next;
 
-      Py_DECREF (obj->symtab);
-      obj->symtab = (symtab_object *) gdbpy_None;
+      gdbpy_ref<> tmp (obj->symtab);
+      obj->symtab = gdbpy_None;
       Py_INCREF (gdbpy_None);
 
       obj->next = NULL;
@@ -518,6 +512,20 @@ del_objfile_sal (struct objfile *objfile, void *datum)
     }
 }
 
+void _initialize_py_symtab ();
+void
+_initialize_py_symtab ()
+{
+  /* Register an objfile "free" callback so we can properly
+     invalidate symbol tables, and symbol table and line data
+     structures when an object file that is about to be
+     deleted.  */
+  stpy_objfile_data_key
+    = register_objfile_data_with_cleanup (NULL, del_objfile_symtab);
+  salpy_objfile_data_key
+    = register_objfile_data_with_cleanup (NULL, del_objfile_sal);
+}
+
 int
 gdbpy_initialize_symtabs (void)
 {
@@ -529,15 +537,6 @@ gdbpy_initialize_symtabs (void)
   if (PyType_Ready (&sal_object_type) < 0)
     return -1;
 
-  /* Register an objfile "free" callback so we can properly
-     invalidate symbol tables, and symbol table and line data
-     structures when an object file that is about to be
-     deleted.  */
-  stpy_objfile_data_key
-    = register_objfile_data_with_cleanup (NULL, del_objfile_symtab);
-  salpy_objfile_data_key
-    = register_objfile_data_with_cleanup (NULL, del_objfile_sal);
-
   if (gdb_pymodule_addobject (gdb_module, "Symtab",
 			      (PyObject *) &symtab_object_type) < 0)
     return -1;
@@ -548,7 +547,7 @@ gdbpy_initialize_symtabs (void)
 
 
 
-static PyGetSetDef symtab_object_getset[] = {
+static gdb_PyGetSetDef symtab_object_getset[] = {
   { "filename", stpy_get_filename, NULL,
     "The symbol table's source filename.", NULL },
   { "objfile", stpy_get_objfile, NULL, "The symtab's objfile.",
@@ -610,7 +609,7 @@ PyTypeObject symtab_object_type = {
   symtab_object_getset		  /*tp_getset */
 };
 
-static PyGetSetDef sal_object_getset[] = {
+static gdb_PyGetSetDef sal_object_getset[] = {
   { "symtab", salpy_get_symtab, NULL, "Symtab object.", NULL },
   { "pc", salpy_get_pc, NULL, "Return the symtab_and_line's pc.", NULL },
   { "last", salpy_get_last, NULL,
