@@ -1,5 +1,6 @@
 /*
- * NVIDIA CUDA Debugger CUDA-GDB Copyright (C) 2013-2021 NVIDIA Corporation
+ * NVIDIA CUDA Debugger CUDA-GDB
+ * Copyright (C) 2013-2022 NVIDIA Corporation
  * Written by CUDA-GDB team at NVIDIA <cudatools@nvidia.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,8 +33,6 @@
 #endif /* __QNXHOST__ */
 #include "gdbsupport/rsp-low.h"
 
-#define TEXTURE_DIM_MAX 4
-
 #ifdef __QNXHOST__
 /* On QNX the packet size is much smaller */
 # undef PBUFSIZ
@@ -48,7 +47,7 @@ extern struct target_waitstatus cuda_last_ws;
 char *buf_head = NULL;
 
 
-char *
+static char *
 append_string (const char *src, char *dest, bool sep)
 {
   char *p;
@@ -66,7 +65,7 @@ append_string (const char *src, char *dest, bool sep)
   return p;
 }
 
-char *
+static char *
 append_bin (const unsigned char *src, char *dest, int size, bool sep)
 {
   char *p;
@@ -84,13 +83,13 @@ append_bin (const unsigned char *src, char *dest, int size, bool sep)
   return p;
 }
 
-char *
+static char *
 extract_string (char *src)
 {
   return strtok (src, ";");
 }
 
-char *
+static char *
 extract_bin (char *src, unsigned char *dest, int size)
 {
   char *p;
@@ -102,7 +101,7 @@ extract_bin (char *src, unsigned char *dest, int size)
   return p;
 }
 
-void
+static void
 cuda_process_suspend_device_packet (char *buf)
 {
   CUDBGResult res;
@@ -114,7 +113,7 @@ cuda_process_suspend_device_packet (char *buf)
 }
 
 
-void
+static void
 cuda_process_resume_device_packet (char *buf)
 {
   CUDBGResult res;
@@ -125,7 +124,7 @@ cuda_process_resume_device_packet (char *buf)
   append_bin ((unsigned char *) &res, buf, sizeof (res), false);
 }
 
-void
+static void
 cuda_process_disassemble_packet (char *buf)
 {
   CUDBGResult res;
@@ -148,7 +147,7 @@ cuda_process_disassemble_packet (char *buf)
   xfree (inst_buf);
 }
 
-void
+static void
 cuda_process_set_breakpoint_packet (char *buf)
 {
   CUDBGResult res;
@@ -161,7 +160,7 @@ cuda_process_set_breakpoint_packet (char *buf)
   append_bin ((unsigned char *) &res, buf, sizeof (res), false);
 }
 
-void
+static void
 cuda_process_unset_breakpoint_packet (char *buf)
 {
   CUDBGResult res;
@@ -175,7 +174,7 @@ cuda_process_unset_breakpoint_packet (char *buf)
   append_bin ((unsigned char *) &res, buf, sizeof (res), false);
 }
 
-void
+static void
 cuda_process_get_adjusted_code_address (char *buf)
 {
   CUDBGResult res;
@@ -194,7 +193,7 @@ cuda_process_get_adjusted_code_address (char *buf)
   p = append_bin ((unsigned char *) &adjusted_addr, p, sizeof (adjusted_addr), false);
 }
 
-void
+static void
 cuda_process_get_host_addr_from_device_addr_packet (char *buf)
 {
   CUDBGResult res;
@@ -211,92 +210,70 @@ cuda_process_get_host_addr_from_device_addr_packet (char *buf)
   p = append_bin ((unsigned char *) &hostaddr, p, sizeof (hostaddr), false);
 }
 
-void
-cuda_process_memcheck_read_error_address_packet (char *buf)
+#ifndef __QNXHOST__
+template <typename TValue>
+using FnGetValueFromAPI = CUDBGResult (*)(uint32_t dev, uint32_t sm, uint32_t wp, TValue *value);
+
+template <typename TValue>
+static void
+cuda_process_update_per_warp_info_in_sm_packet (char *buf, FnGetValueFromAPI<TValue>&& get_value)
 {
   CUDBGResult res;
   char *p;
   uint32_t dev;
   uint32_t sm;
   uint32_t wp;
-  uint32_t ln;
-  uint64_t address;
-  ptxStorageKind storage;
+  uint64_t valid_warps_mask = 0;
+  uint32_t num_warps;
+  TValue value;
 
   extract_bin (NULL, (unsigned char *) &dev, sizeof (dev));
   extract_bin (NULL, (unsigned char *) &sm,  sizeof (sm));
-  extract_bin (NULL, (unsigned char *) &wp,  sizeof (wp));
-  extract_bin (NULL, (unsigned char *) &ln,  sizeof (ln));
+  extract_bin (NULL, (unsigned char *) &num_warps, sizeof (num_warps));
 
-  res = cudbgAPI->memcheckReadErrorAddress (dev, sm, wp, ln, &address, &storage);
-  p = append_bin ((unsigned char *) &res, buf, sizeof (res), true);
-  p = append_bin ((unsigned char *) &address, p, sizeof (address), true);
-  p = append_bin ((unsigned char *) &storage, p, sizeof (storage), false);
+  res = cudbgAPI->readValidWarps (dev, sm, &valid_warps_mask);
+  p = append_bin ((unsigned char *) &valid_warps_mask, buf, sizeof (valid_warps_mask), true);
+  for (wp = 0; wp < num_warps; wp++)
+    {
+      if (valid_warps_mask & (1ULL << wp))
+        {
+          if (res == CUDBG_SUCCESS)
+            res = get_value (dev, sm, wp, &value);
+          p = append_bin ((unsigned char *) &value, p, sizeof (value), true);
+        }
+    }
+  p = append_bin ((unsigned char *) &res, p, sizeof (res), false);
 }
 
-#ifndef __QNXHOST__
-void
+static void
 cuda_process_update_grid_id_in_sm_packet (char *buf)
 {
-  CUDBGResult res;
-  char *p;
-  uint32_t dev;
-  uint32_t sm;
-  uint32_t wp;
-  uint64_t valid_warps_mask = 0;
-  uint32_t num_warps;
-  uint64_t grid_id;
-
-  extract_bin (NULL, (unsigned char *) &dev, sizeof (dev));
-  extract_bin (NULL, (unsigned char *) &sm,  sizeof (sm));
-  extract_bin (NULL, (unsigned char *) &num_warps, sizeof (num_warps));
-
-  res = cudbgAPI->readValidWarps (dev, sm, &valid_warps_mask);
-  p = append_bin ((unsigned char *) &valid_warps_mask, buf, sizeof (valid_warps_mask), true);
-  for (wp = 0; wp < num_warps; wp++)
-    {
-      if (valid_warps_mask & (1ULL << wp))
-        {
-          if (res == CUDBG_SUCCESS)
-            res = cudbgAPI->readGridId (dev, sm, wp, &grid_id);
-          p = append_bin ((unsigned char *) &grid_id, p, sizeof (grid_id), true);
-        }
-    }
-  p = append_bin ((unsigned char *) &res, p, sizeof (res), false);
+  cuda_process_update_per_warp_info_in_sm_packet<uint64_t> (
+    buf, [](uint32_t dev, uint32_t sm, uint32_t wp, uint64_t *gridId64) {
+      return cudbgAPI->readGridId (dev, sm, wp, gridId64);
+    });
 }
 
-void
+static void
+cuda_process_update_cluster_idx_in_sm_packet (char *buf)
+{
+  cuda_process_update_per_warp_info_in_sm_packet<CuDim3> (
+    buf, [](uint32_t dev, uint32_t sm, uint32_t wp, CuDim3 *cluster_idx) {
+      return cudbgAPI->readClusterIdx (dev, sm, wp, cluster_idx);
+    });
+}
+
+static void
 cuda_process_update_block_idx_in_sm_packet (char *buf)
 {
-  CUDBGResult res;
-  char *p;
-  uint32_t dev;
-  uint32_t sm;
-  uint32_t wp;
-  uint64_t valid_warps_mask = 0;
-  uint32_t num_warps;
-  CuDim3 block_idx;
-
-  extract_bin (NULL, (unsigned char *) &dev, sizeof (dev));
-  extract_bin (NULL, (unsigned char *) &sm, sizeof (sm));
-  extract_bin (NULL, (unsigned char *) &num_warps, sizeof (num_warps));
-
-  res = cudbgAPI->readValidWarps (dev, sm, &valid_warps_mask);
-  p = append_bin ((unsigned char *) &valid_warps_mask, buf, sizeof (valid_warps_mask), true);
-  for (wp = 0; wp < num_warps; wp++)
-    {
-      if (valid_warps_mask & (1ULL << wp))
-        {
-          if (res == CUDBG_SUCCESS)
-            res = cudbgAPI->readBlockIdx (dev, sm, wp, &block_idx);
-          p = append_bin ((unsigned char *) &block_idx, p, sizeof (block_idx), true);
-        }
-    }
-  p = append_bin ((unsigned char *) &res, p, sizeof (res), false);
+  cuda_process_update_per_warp_info_in_sm_packet<CuDim3> (
+    buf, [](uint32_t dev, uint32_t sm, uint32_t wp, CuDim3 *block_idx) {
+      return cudbgAPI->readBlockIdx (dev, sm, wp, block_idx);
+    });
 }
 #endif
 
-void
+static void
 cuda_process_update_thread_idx_in_warp_packet (char *buf)
 {
   CUDBGResult res;
@@ -328,7 +305,7 @@ cuda_process_update_thread_idx_in_warp_packet (char *buf)
   p = append_bin ((unsigned char *) &res, p, sizeof (res), false);
 }
 
-void
+static void
 cuda_process_notification_analyze_packet (char *buf)
 {
   int trap_expected;
@@ -346,7 +323,7 @@ cuda_process_notification_analyze_packet (char *buf)
   append_string ("OK", buf, false);
 }
 
-void
+static void
 cuda_process_notification_received_packet (char *buf)
 {
   bool received;
@@ -355,7 +332,7 @@ cuda_process_notification_received_packet (char *buf)
   append_bin ((unsigned char *) &received, buf, sizeof (received), false);
 }
 
-void
+static void
 cuda_process_notification_pending_packet (char *buf)
 {
   bool pending;
@@ -364,21 +341,21 @@ cuda_process_notification_pending_packet (char *buf)
   append_bin ((unsigned char *) &pending, buf, sizeof (pending), false);
 }
 
-void
+static void
 cuda_process_notification_mark_consumed_packet (char *buf)
 {
   cuda_notification_mark_consumed ();
   append_string ("OK", buf, false);
 }
 
-void
+static void
 cuda_process_notification_consume_pending_packet (char *buf)
 {
   cuda_notification_consume_pending ();
   append_string ("OK", buf, false);
 }
 
-void
+static void
 cuda_process_notification_aliased_event_packet (char *buf)
 {
   bool aliased_event;
@@ -389,7 +366,7 @@ cuda_process_notification_aliased_event_packet (char *buf)
   append_bin ((unsigned char *) &aliased_event, buf, sizeof (aliased_event), false);
 }
 
-void
+static void
 cuda_process_single_step_warp_packet (char *buf)
 {
   CUDBGResult res;
@@ -411,7 +388,7 @@ cuda_process_single_step_warp_packet (char *buf)
 }
 
 #ifdef __QNXHOST__
-void
+static void
 cuda_process_set_symbols (char *buf)
 {
   bool symbols_are_set = false;
@@ -442,13 +419,16 @@ cuda_process_set_symbols (char *buf)
 }
 #endif /* __QNXHOST__ */
 
-void
+static void
 cuda_process_initialize_target_packet (char *buf)
 {
   char *p;
   bool driver_is_compatible;
+  bool cuda_memcheck;
 
   extract_bin (NULL, (unsigned char *) &cuda_software_preemption, sizeof (cuda_software_preemption));
+  /* CUDA MEMCHECK support is removed from CUDA GDB: this field is left to maintain
+   * the binary compatibility with legacy CUDA GDB server binaries */
   extract_bin (NULL, (unsigned char *) &cuda_memcheck, sizeof (cuda_memcheck));
   extract_bin (NULL, (unsigned char *) &cuda_launch_blocking, sizeof (cuda_launch_blocking));
 
@@ -462,7 +442,7 @@ cuda_process_initialize_target_packet (char *buf)
   p = append_bin ((unsigned char *) &driver_is_compatible, p, sizeof (driver_is_compatible), false);
 }
 
-void
+static void
 cuda_process_get_num_devices_packet (char *buf)
 {
   CUDBGResult res;
@@ -474,7 +454,7 @@ cuda_process_get_num_devices_packet (char *buf)
   p = append_bin ((unsigned char *) &num_dev, p, sizeof (num_dev), false);
 }
 
-void
+static void
 cuda_process_query_device_spec_packet (char *buf)
 {
   char *p;
@@ -510,7 +490,7 @@ cuda_process_query_device_spec_packet (char *buf)
   p = append_string (sm_type, p, false);
 }
 
-void
+static void
 cuda_process_is_device_code_address_packet (char *buf)
 {
   CUDBGResult res;
@@ -524,7 +504,7 @@ cuda_process_is_device_code_address_packet (char *buf)
   p = append_bin ((unsigned char *) &is_device_address, p, sizeof (is_device_address), false);
 }
 
-void
+static void
 cuda_process_get_grid_status_packet (char *buf)
 {
   CUDBGResult res;
@@ -541,7 +521,7 @@ cuda_process_get_grid_status_packet (char *buf)
   p = append_bin ((unsigned char *) &status, p, sizeof (status), false);
 }
 
-void
+static void
 cuda_process_get_grid_info_packet (char *buf)
 {
   CUDBGResult res;
@@ -558,7 +538,7 @@ cuda_process_get_grid_info_packet (char *buf)
   p = append_bin ((unsigned char *) &info, p, sizeof (info), false);
 }
 
-void
+static void
 cuda_process_read_grid_id_packet (char *buf)
 {
   CUDBGResult res;
@@ -577,7 +557,26 @@ cuda_process_read_grid_id_packet (char *buf)
   p = append_bin ((unsigned char *) &grid_id, p, sizeof (grid_id), false);
 }
 
-void
+static void
+cuda_process_read_cluster_idx_packet (char *buf)
+{
+  CUDBGResult res;
+  char *p;
+  uint32_t dev;
+  uint32_t sm;
+  uint32_t wp;
+  CuDim3 cluster_idx;
+
+  extract_bin (NULL, (unsigned char *) &dev, sizeof (dev));
+  extract_bin (NULL, (unsigned char *) &sm,  sizeof (sm));
+  extract_bin (NULL, (unsigned char *) &wp,  sizeof (wp));
+
+  res = cudbgAPI->readClusterIdx (dev, sm, wp, &cluster_idx);  
+  p = append_bin ((unsigned char *) &res, buf, sizeof (res), true);
+  p = append_bin ((unsigned char *) &cluster_idx, p, sizeof (cluster_idx), false);
+}
+
+static void
 cuda_process_read_block_idx_packet (char *buf)
 {
   CUDBGResult res;
@@ -596,7 +595,7 @@ cuda_process_read_block_idx_packet (char *buf)
   p = append_bin ((unsigned char *) &block_idx, p, sizeof (block_idx), false);
 }
 
-void
+static void
 cuda_process_read_thread_idx_packet (char *buf)
 {
   CUDBGResult res;
@@ -617,7 +616,7 @@ cuda_process_read_thread_idx_packet (char *buf)
   p = append_bin ((unsigned char *) &thread_idx, p, sizeof (thread_idx), false);
 }
 
-void
+static void
 cuda_process_read_broken_warps_packet (char *buf)
 {
   CUDBGResult res;
@@ -634,7 +633,7 @@ cuda_process_read_broken_warps_packet (char *buf)
   p = append_bin ((unsigned char *) &broken_warps_mask, p, sizeof (broken_warps_mask), false);
 }
 
-void
+static void
 cuda_process_read_valid_warps_packet (char *buf)
 {
   CUDBGResult res;
@@ -651,7 +650,7 @@ cuda_process_read_valid_warps_packet (char *buf)
   p = append_bin ((unsigned char *) &valid_warps_mask, p, sizeof (valid_warps_mask), false);
 }
 
-void
+static void
 cuda_process_read_valid_lanes_packet (char *buf)
 {
   CUDBGResult res;
@@ -670,7 +669,7 @@ cuda_process_read_valid_lanes_packet (char *buf)
   p = append_bin ((unsigned char *) &valid_lanes_mask, p, sizeof (valid_lanes_mask), false);
 }
 
-void
+static void
 cuda_process_read_active_lanes_packet (char *buf)
 {
   CUDBGResult res;
@@ -689,7 +688,7 @@ cuda_process_read_active_lanes_packet (char *buf)
   p = append_bin ((unsigned char *) &active_lanes_mask, p, sizeof (active_lanes_mask), false);
 }
 
-void
+static void
 cuda_process_read_virtual_pc_packet (char *buf)
 {
   CUDBGResult res;
@@ -710,7 +709,7 @@ cuda_process_read_virtual_pc_packet (char *buf)
   p = append_bin ((unsigned char *) &value, p, sizeof (value), false);
 }
 
-void
+static void
 cuda_process_read_pc_packet (char *buf)
 {
   CUDBGResult res;
@@ -731,7 +730,7 @@ cuda_process_read_pc_packet (char *buf)
   p = append_bin ((unsigned char *) &value, p, sizeof (value), false);
 }
 
-void
+static void
 cuda_process_read_register_packet (char *buf)
 {
   CUDBGResult res;
@@ -755,7 +754,7 @@ cuda_process_read_register_packet (char *buf)
 }
 
 
-void
+static void
 cuda_process_read_lane_exception_packet (char *buf)
 {
   CUDBGResult res;
@@ -776,7 +775,7 @@ cuda_process_read_lane_exception_packet (char *buf)
   p = append_bin ((unsigned char *) &exception, p, sizeof (exception), false);
 }
 
-void
+static void
 cuda_process_read_call_depth_packet (char *buf)
 {
   CUDBGResult res;
@@ -797,7 +796,7 @@ cuda_process_read_call_depth_packet (char *buf)
   p = append_bin ((unsigned char *) &value, p, sizeof (value), false);;
 }
 
-void
+static void
 cuda_process_read_syscall_call_depth_packet (char *buf)
 {
   CUDBGResult res;
@@ -818,7 +817,7 @@ cuda_process_read_syscall_call_depth_packet (char *buf)
   p = append_bin ((unsigned char *) &value, p, sizeof (value), false);;
 }
 
-void
+static void
 cuda_process_read_virtual_return_address_packet (char *buf)
 {
   CUDBGResult res;
@@ -841,7 +840,7 @@ cuda_process_read_virtual_return_address_packet (char *buf)
   p = append_bin ((unsigned char *) &value, p, sizeof (value), false);
 }
 
-void
+static void
 cuda_process_read_code_memory_packet (char *buf)
 {
   CUDBGResult res;
@@ -862,7 +861,7 @@ cuda_process_read_code_memory_packet (char *buf)
   xfree (value);
 }
 
-void
+static void
 cuda_process_read_const_memory_packet (char *buf)
 {
   CUDBGResult res;
@@ -883,7 +882,7 @@ cuda_process_read_const_memory_packet (char *buf)
   xfree (value);
 }
 
-void
+static void
 cuda_process_read_generic_memory_packet (char *buf)
 {
   CUDBGResult res;
@@ -910,7 +909,7 @@ cuda_process_read_generic_memory_packet (char *buf)
   xfree (value);
 }
 
-void
+static void
 cuda_process_read_pinned_memory_packet (char *buf)
 {
   CUDBGResult res;
@@ -929,7 +928,7 @@ cuda_process_read_pinned_memory_packet (char *buf)
   xfree (value);
 }
 
-void
+static void
 cuda_process_read_param_memory_packet (char *buf)
 {
   CUDBGResult res;
@@ -954,7 +953,7 @@ cuda_process_read_param_memory_packet (char *buf)
   xfree (value);
 }
 
-void
+static void
 cuda_process_read_shared_memory_packet (char *buf)
 {
   CUDBGResult res;
@@ -979,65 +978,7 @@ cuda_process_read_shared_memory_packet (char *buf)
   xfree (value);
 }
 
-void
-cuda_process_read_texture_memory_packet (char *buf)
-{
-  CUDBGResult res;
-  char *p;
-  uint32_t dev;
-  uint32_t sm;
-  uint32_t wp;
-  uint32_t id;
-  uint32_t dim;
-  uint32_t coords[TEXTURE_DIM_MAX];
-  uint32_t sz;
-  void *value;
-
-  extract_bin (NULL, (unsigned char *) &dev,   sizeof (dev));
-  extract_bin (NULL, (unsigned char *) &sm,    sizeof (sm));
-  extract_bin (NULL, (unsigned char *) &wp,    sizeof (wp));
-  extract_bin (NULL, (unsigned char *) &id,    sizeof (id));
-  extract_bin (NULL, (unsigned char *) &dim,   sizeof (dim));
-  extract_bin (NULL, (unsigned char *) coords, sizeof (coords[0]) * TEXTURE_DIM_MAX);
-  extract_bin (NULL, (unsigned char *) &sz,    sizeof (sz));
-
-  value = xmalloc (sz);
-  res = cudbgAPI->readTextureMemory (dev, sm, wp, id, dim, coords, value, sz);
-  p = append_bin ((unsigned char *) &res, buf, sizeof (res), true);
-  p = append_bin ((unsigned char *) value, p, sz, false);
-  xfree (value);
-}
-
-void
-cuda_process_read_texture_memory_bindless_packet (char *buf)
-{
-  CUDBGResult res;
-  char *p;
-  uint32_t dev;
-  uint32_t sm;
-  uint32_t wp;
-  uint32_t tex_symtab_index;
-  uint32_t dim;
-  uint32_t coords[TEXTURE_DIM_MAX];
-  uint32_t sz;
-  void *value;
-
-  extract_bin (NULL, (unsigned char *) &dev,   sizeof (dev));
-  extract_bin (NULL, (unsigned char *) &sm,    sizeof (sm));
-  extract_bin (NULL, (unsigned char *) &wp,    sizeof (wp));
-  extract_bin (NULL, (unsigned char *) &tex_symtab_index, sizeof (tex_symtab_index));
-  extract_bin (NULL, (unsigned char *) &dim,   sizeof (dim));
-  extract_bin (NULL, (unsigned char *) coords, sizeof (coords[0]) * TEXTURE_DIM_MAX);
-  extract_bin (NULL, (unsigned char *) &sz,    sizeof (sz));
-
-  value = xmalloc (sz);
-  res = cudbgAPI->readTextureMemoryBindless (dev, sm, wp, tex_symtab_index, dim, coords, value, sz);
-  p = append_bin ((unsigned char *) &res, buf, sizeof (res), true);
-  p = append_bin ((unsigned char *) value, p, sz, false);
-  xfree (value);
-}
-
-void
+static void
 cuda_process_read_local_memory_packet (char *buf)
 {
   CUDBGResult res;
@@ -1064,7 +1005,7 @@ cuda_process_read_local_memory_packet (char *buf)
   xfree (value);
 }
 
-void
+static void
 cuda_process_write_generic_memory_packet (char *buf)
 {
   CUDBGResult res;
@@ -1091,7 +1032,7 @@ cuda_process_write_generic_memory_packet (char *buf)
   xfree (value);
 }
 
-void
+static void
 cuda_process_write_pinned_memory_packet (char *buf)
 {
   CUDBGResult res;
@@ -1109,7 +1050,7 @@ cuda_process_write_pinned_memory_packet (char *buf)
   xfree (value);
 }
 
-void
+static void
 cuda_process_write_param_memory_packet (char *buf)
 {
   CUDBGResult res;
@@ -1134,7 +1075,7 @@ cuda_process_write_param_memory_packet (char *buf)
   xfree (value);
 }
 
-void
+static void
 cuda_process_write_shared_memory_packet (char *buf)
 {
   CUDBGResult res;
@@ -1159,7 +1100,7 @@ cuda_process_write_shared_memory_packet (char *buf)
   xfree (value);
 }
 
-void
+static void
 cuda_process_write_local_memory_packet (char *buf)
 {
   CUDBGResult res;
@@ -1186,7 +1127,7 @@ cuda_process_write_local_memory_packet (char *buf)
   xfree (value);
 }
 
-void
+static void
 cuda_process_write_register_packet (char *buf)
 {
   CUDBGResult res;
@@ -1208,7 +1149,7 @@ cuda_process_write_register_packet (char *buf)
   append_bin ((unsigned char *) &res, buf, sizeof (res), false);
 }
 
-void
+static void
 cuda_process_check_pending_sigint_packet (char *buf)
 {
   bool ret_val;
@@ -1222,7 +1163,7 @@ cuda_process_check_pending_sigint_packet (char *buf)
   append_bin ((unsigned char *) &ret_val, buf, sizeof (ret_val), false);
 }
 
-void
+static void
 cuda_process_api_initialize_packet (char *buf)
 {
   CUDBGResult res;
@@ -1231,7 +1172,7 @@ cuda_process_api_initialize_packet (char *buf)
   append_bin ((unsigned char *) &res, buf, sizeof (res), false);
 }
 
-void
+static void
 cuda_process_api_finalize_packet (char *buf)
 {
   CUDBGResult res;
@@ -1245,7 +1186,7 @@ cuda_process_api_finalize_packet (char *buf)
   append_bin ((unsigned char *) &res, buf, sizeof (res), false);
 }
 
-void
+static void
 cuda_process_api_request_clear_attach_state (char *buf)
 {
   CUDBGResult res;
@@ -1253,7 +1194,7 @@ cuda_process_api_request_clear_attach_state (char *buf)
   append_bin ((unsigned char *) &res, buf, sizeof (res), false);
 }
 
-void
+static void
 cuda_process_api_request_cleanup_on_detach_packet (char *buf)
 {
   CUDBGResult res;
@@ -1265,7 +1206,7 @@ cuda_process_api_request_cleanup_on_detach_packet (char *buf)
   append_bin ((unsigned char *) &res, buf, sizeof (res), false);
 }
 
-void
+static void
 cuda_process_set_option_packet (char *buf)
 {
   const char *stop_signal_str = NULL;
@@ -1282,7 +1223,7 @@ cuda_process_set_option_packet (char *buf)
   append_string ("OK", buf, false);
 }
 
-void
+static void
 cuda_process_set_async_launch_notifications (char *buf)
 {
   CUDBGResult res;
@@ -1293,7 +1234,7 @@ cuda_process_set_async_launch_notifications (char *buf)
   append_bin ((unsigned char *) &res, buf, sizeof (res), false);
 }
 
-void
+static void
 cuda_process_api_read_device_exception_state (char *buf)
 {
   CUDBGResult res;
@@ -1311,7 +1252,7 @@ cuda_process_api_read_device_exception_state (char *buf)
   xfree (value);
 }
 
-void
+static void
 cuda_process_query_trace_message (char *buf)
 {
   struct cuda_trace_msg *msg;
@@ -1331,7 +1272,7 @@ cuda_process_query_trace_message (char *buf)
 }
 
 #ifdef __QNXHOST__
-void
+static void
 cuda_process_version_handshake (char *buf)
 {
   char str[256];
@@ -1408,11 +1349,11 @@ handle_cuda_packet (char *buf)
     case READ_SHARED_MEMORY:
       cuda_process_read_shared_memory_packet (buf);
       break;
+    /* Texture support has been removed - let the other
+     * side know we no longer support it. */
     case READ_TEXTURE_MEMORY:
-      cuda_process_read_texture_memory_packet (buf);
-      break;
     case READ_TEXTURE_MEMORY_BINDLESS:
-      cuda_process_read_texture_memory_bindless_packet (buf);
+      buf[0] = '\0';
       break;
     case READ_LOCAL_MEMORY:
       cuda_process_read_local_memory_packet (buf);
@@ -1462,8 +1403,10 @@ handle_cuda_packet (char *buf)
     case DISASSEMBLE:
       cuda_process_disassemble_packet (buf);
       break;
+    /* CUDA MEMCHECK support is removed from CUDA GDB: this field is left
+     * to maintain the binary compatibility with legacy CUDA GDB server
+     * binaries */
     case MEMCHECK_READ_ERROR_ADDRESS:
-      cuda_process_memcheck_read_error_address_packet (buf);
       break;
     case GET_NUM_DEVICES:
       cuda_process_get_num_devices_packet (buf);
@@ -1546,12 +1489,20 @@ handle_cuda_packet (char *buf)
       break;
     case READ_DEVICE_EXCEPTION_STATE:
       cuda_process_api_read_device_exception_state (buf);
-#ifdef __QNXHOST__
       break;
+#ifdef __QNXHOST__
     case VERSION_HANDSHAKE:
       cuda_process_version_handshake (buf);
-#endif /* __QNXHOST__ */
       break;
+#endif /* __QNXHOST__ */
+    case READ_CLUSTER_IDX:
+      cuda_process_read_cluster_idx_packet (buf);
+      break;
+#ifndef __QNXHOST__
+    case UPDATE_CLUSTER_IDX_IN_SM:
+      cuda_process_update_cluster_idx_in_sm_packet (buf);
+      break;
+#endif /* !__QNXHOST__ */
     default:
       error ("unknown cuda packet.\n");
       break;
