@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2019 NVIDIA CORPORATION. All rights reserved.
+ * Copyright (c) 2014-2022 NVIDIA CORPORATION. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -56,11 +56,11 @@ static CUDBGResult determineCTAId(uint32_t dev, uint32_t sm, uint32_t wp, uint32
 	CudbgCTATableEntry *ctate, *ctate1;
 	uint32_t ctateIdx = 0;
 
-	GET_TABLE_ENTRY(ctate, CUDBG_ERROR_UNKNOWN,
+	GET_TABLE_ENTRY(ctate, NULL, CUDBG_ERROR_UNKNOWN,
 			"wp%u_sm%u_dev%u_cta", wp, sm, dev);
 
 	do {
-		ctate1 = cuCoreGetMapEntry(&curcc->tableEntriesMap, "cta%u_sm%u_dev%u", ctateIdx++, sm, dev);
+		ctate1 = cuCoreGetMapEntry(&curcc->tableEntriesMap, NULL, "cta%u_sm%u_dev%u", ctateIdx++, sm, dev);
 	} while (ctate1 != NULL && ctate1 != ctate);
 
 	*cta = ctateIdx-1;
@@ -72,10 +72,10 @@ static CUDBGResult getGridId(uint32_t dev, uint32_t sm, uint32_t wp, uint64_t *g
 	CudbgCTATableEntry *ctate;
 	CudbgWarpTableEntry *wte;
 
-	GET_TABLE_ENTRY(wte, CUDBG_ERROR_INVALID_WARP,
+	GET_TABLE_ENTRY(wte, NULL, CUDBG_ERROR_INVALID_WARP,
 			"wp%u_sm%u_dev%u", wp, sm, dev);
 
-	GET_TABLE_ENTRY(ctate, CUDBG_ERROR_UNKNOWN,
+	GET_TABLE_ENTRY(ctate, NULL, CUDBG_ERROR_UNKNOWN,
 			"wp%u_sm%u_dev%u_cta", wp, sm, dev);
 
 	*gridId = ctate->gridId64;
@@ -116,10 +116,10 @@ DEF_API_CALL(getElfImage)(uint32_t dev, uint32_t sm, uint32_t wp,
 	if ((rc = getGridId(dev, sm, wp, &gridId)) != CUDBG_SUCCESS)
 		return rc;
 
-	GET_TABLE_ENTRY(gte, CUDBG_ERROR_INVALID_GRID,
+	GET_TABLE_ENTRY(gte, NULL, CUDBG_ERROR_INVALID_GRID,
 			"grid%llu_dev%u", gridId, dev);
 
-	GET_TABLE_ENTRY(scn, CUDBG_ERROR_INVALID_MODULE,
+	GET_TABLE_ENTRY(scn, NULL, CUDBG_ERROR_INVALID_MODULE,
 			"%celf_handle%llx",
 			relocated ? 'r' : 'u', gte->moduleHandle);
 
@@ -151,7 +151,7 @@ DEF_API_CALL(getNumSMs)(uint32_t dev, uint32_t *numSMs)
 
 	VERIFY_ARG(numSMs);
 
-	GET_TABLE_ENTRY(dte, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
+	GET_TABLE_ENTRY(dte, NULL, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
 
 	*numSMs = dte->numSMs;
 
@@ -166,7 +166,7 @@ DEF_API_CALL(getNumWarps)(uint32_t dev, uint32_t *numWarps)
 
 	VERIFY_ARG(numWarps);
 
-	GET_TABLE_ENTRY(dte, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
+	GET_TABLE_ENTRY(dte, NULL, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
 
 	*numWarps = dte->numWarpsPerSM;
 
@@ -181,7 +181,7 @@ DEF_API_CALL(getNumLanes)(uint32_t dev, uint32_t *numLanes)
 
 	VERIFY_ARG(numLanes);
 
-	GET_TABLE_ENTRY(dte, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
+	GET_TABLE_ENTRY(dte, NULL, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
 
 	*numLanes = dte->numLanesPerWarp;
 
@@ -196,7 +196,7 @@ DEF_API_CALL(getNumRegisters)(uint32_t dev, uint32_t *numRegs)
 
 	VERIFY_ARG(numRegs);
 
-	GET_TABLE_ENTRY(dte, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
+	GET_TABLE_ENTRY(dte, NULL, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
 
 	*numRegs = dte->numRegsPerLane;
 
@@ -306,7 +306,7 @@ DEF_API_CALL(readSharedMemory)(uint32_t dev, uint32_t sm, uint32_t wp,
 	if (rc != CUDBG_SUCCESS)
 		return rc;
 
-	GET_TABLE_ENTRY(scn, CUDBG_ERROR_MISSING_DATA,
+	GET_TABLE_ENTRY(scn, NULL, CUDBG_ERROR_MISSING_DATA,
 			"cta%u_sm%u_dev%u_shared", ctaId, sm, dev);
 
 	if (cuCoreReadSectionData(curcc->e, scn, &data) != 0)
@@ -334,7 +334,7 @@ DEF_API_CALL(readLocalMemory)(uint32_t dev, uint32_t sm, uint32_t wp,
 
 	VERIFY_ARG(buf);
 
-	GET_TABLE_ENTRY(scn, CUDBG_ERROR_MISSING_DATA,
+	GET_TABLE_ENTRY(scn, NULL, CUDBG_ERROR_MISSING_DATA,
 			"ln%u_wp%u_sm%u_dev%u_local", ln, wp, sm, dev);
 
 	if (cuCoreReadSectionData(curcc->e, scn, &data) != 0)
@@ -371,10 +371,10 @@ DEF_API_CALL(readGenericMemory)(uint32_t dev, uint32_t sm, uint32_t wp,
 	if ((rc = getGridId(dev, sm, wp, &gridId)) != CUDBG_SUCCESS)
 		return rc;
 
-	GET_TABLE_ENTRY(gte, CUDBG_ERROR_INVALID_GRID,
+	GET_TABLE_ENTRY(gte, NULL, CUDBG_ERROR_INVALID_GRID,
 			"grid%llu_dev%u", gridId, dev);
 
-	GET_TABLE_ENTRY(cte, CUDBG_ERROR_INVALID_CONTEXT,
+	GET_TABLE_ENTRY(cte, NULL, CUDBG_ERROR_INVALID_CONTEXT,
 			"ctx%llu_dev%u", gte->contextId, dev);
 
 	TRACE_FUNC("dev=%u sm=%u wp=%u ln=%u addr=0x%llx buf=%p sz=%u lmemBase=0x%llx smemBase=0x%llx",
@@ -411,10 +411,10 @@ DEF_API_CALL(readParamMemory)(uint32_t dev, uint32_t sm, uint32_t wp,
 	if ((rc = getGridId(dev, sm, wp, &gridId)) != CUDBG_SUCCESS)
 		return rc;
 
-	GET_TABLE_ENTRY(gte, CUDBG_ERROR_INVALID_GRID,
+	GET_TABLE_ENTRY(gte, NULL, CUDBG_ERROR_INVALID_GRID,
 			"grid%llu_dev%u", gridId, dev);
 
-	GET_TABLE_ENTRY(scn, CUDBG_ERROR_UNKNOWN,
+	GET_TABLE_ENTRY(scn, NULL, CUDBG_ERROR_UNKNOWN,
 			"grid%llu_dev%u_param", gridId, dev);
 
 	if (cuCoreReadSectionData(curcc->e, scn, &data) != 0)
@@ -451,23 +451,30 @@ DEF_API_CALL(readWarpState)(uint32_t devId, uint32_t sm, uint32_t wp,
 	CudbgCTATableEntry *ctate;
 	CudbgThreadTableEntry *tte;
 	CUDBGResult rc;
+	size_t ctateSize;
 
 	TRACE_FUNC("devId=%u sm=%u wp=%u state=%p", devId, sm, wp, state);
 
 	VERIFY_ARG(state);
 
-	GET_TABLE_ENTRY(wte, CUDBG_ERROR_INVALID_WARP,
+	GET_TABLE_ENTRY(wte, NULL, CUDBG_ERROR_INVALID_WARP,
 			"wp%u_sm%u_dev%u", wp, sm, devId);
 
-	GET_TABLE_ENTRY(ctate, CUDBG_ERROR_UNKNOWN,
+	GET_TABLE_ENTRY(ctate, &ctateSize, CUDBG_ERROR_UNKNOWN,
 			"wp%u_sm%u_dev%u_cta", wp, sm, devId);
 
-	GET_TABLE_ENTRY(gte, CUDBG_ERROR_INVALID_GRID,
+	GET_TABLE_ENTRY(gte, NULL, CUDBG_ERROR_INVALID_GRID,
 			"grid%llu_dev%u", ctate->gridId64, devId);
 
 	memset(state, 0, sizeof(*state));
 	state->gridId = gte->gridId64;
 	state->errorPC = wte->errorPC;
+	if (offsetof(CudbgCTATableEntry, clusterIdxZ) < ctateSize)
+	{
+		state->clusterIdx.x = ctate->clusterIdxX;
+		state->clusterIdx.y = ctate->clusterIdxY;
+		state->clusterIdx.z = ctate->clusterIdxZ;
+	}
 	state->blockIdx.x = ctate->blockIdxX;
 	state->blockIdx.y = ctate->blockIdxY;
 	state->blockIdx.z = ctate->blockIdxZ;
@@ -486,7 +493,7 @@ DEF_API_CALL(readWarpState)(uint32_t devId, uint32_t sm, uint32_t wp,
 			continue;
 
 		/* For every valid lane there must be a corresponding ThreadTableEntry */
-		GET_TABLE_ENTRY(tte, CUDBG_ERROR_INTERNAL,
+		GET_TABLE_ENTRY(tte, NULL, CUDBG_ERROR_INTERNAL,
 				"ln%u_wp%u_sm%u_dev%u", ln, wp, sm, devId);
 
 		state->lane[ln].virtualPC = tte->virtualPC;
@@ -509,7 +516,7 @@ DEF_API_CALL(readThreadIdx)(uint32_t dev, uint32_t sm, uint32_t wp,
 
 	VERIFY_ARG(threadIdx);
 
-	GET_TABLE_ENTRY(tte, CUDBG_ERROR_INVALID_LANE,
+	GET_TABLE_ENTRY(tte, NULL, CUDBG_ERROR_INVALID_LANE,
 			"ln%u_wp%u_sm%u_dev%u", ln, wp, sm, dev);
 
 	threadIdx->x = tte->threadIdxX;
@@ -528,7 +535,7 @@ DEF_API_CALL(readVirtualPC)(uint32_t dev, uint32_t sm, uint32_t wp,
 
 	VERIFY_ARG(pc);
 
-	GET_TABLE_ENTRY(tte, CUDBG_ERROR_INVALID_LANE,
+	GET_TABLE_ENTRY(tte, NULL, CUDBG_ERROR_INVALID_LANE,
 			"ln%u_wp%u_sm%u_dev%u", ln, wp, sm, dev);
 
 	*pc = tte->virtualPC;
@@ -550,10 +557,10 @@ DEF_API_CALL(getTID)(uint32_t dev, uint32_t sm, uint32_t wp, uint32_t *tid)
 	if ((rc = getGridId(dev, sm, wp, &gridId)) != CUDBG_SUCCESS)
 		return rc;
 
-	GET_TABLE_ENTRY(gte, CUDBG_ERROR_INVALID_GRID,
+	GET_TABLE_ENTRY(gte, NULL, CUDBG_ERROR_INVALID_GRID,
 			"grid%llu_dev%u", gridId, dev);
 
-	GET_TABLE_ENTRY(cte, CUDBG_ERROR_INVALID_CONTEXT,
+	GET_TABLE_ENTRY(cte, NULL, CUDBG_ERROR_INVALID_CONTEXT,
 			"ctx%llu_dev%u", gte->contextId, dev);
 
 	*tid = cte->tid;
@@ -571,10 +578,10 @@ DEF_API_CALL(readBlockIdx)(uint32_t dev, uint32_t sm, uint32_t wp,
 
 	VERIFY_ARG(blockIdx);
 
-	GET_TABLE_ENTRY(wte, CUDBG_ERROR_INVALID_WARP,
+	GET_TABLE_ENTRY(wte, NULL, CUDBG_ERROR_INVALID_WARP,
 			"wp%u_sm%u_dev%u", wp, sm, dev);
 
-	GET_TABLE_ENTRY(ctate, CUDBG_ERROR_UNKNOWN,
+	GET_TABLE_ENTRY(ctate, NULL, CUDBG_ERROR_UNKNOWN,
 			"wp%u_sm%u_dev%u_cta", wp, sm, dev);
 
 	blockIdx->x = ctate->blockIdxX;
@@ -598,7 +605,7 @@ DEF_API_CALL(getBlockDim)(uint32_t dev, uint32_t sm, uint32_t wp,
 	if ((rc = getGridId(dev, sm, wp, &gridId)) != CUDBG_SUCCESS)
 		return rc;
 
-	GET_TABLE_ENTRY(gte, CUDBG_ERROR_INVALID_GRID,
+	GET_TABLE_ENTRY(gte, NULL, CUDBG_ERROR_INVALID_GRID,
 			"grid%llu_dev%u", gridId, dev);
 
 	blockDim->x = gte->blockDimX;
@@ -622,7 +629,7 @@ DEF_API_CALL(getGridDim)(uint32_t dev, uint32_t sm, uint32_t wp,
 	if ((rc = getGridId(dev, sm, wp, &gridId)) != CUDBG_SUCCESS)
 		return rc;
 
-	GET_TABLE_ENTRY(gte, CUDBG_ERROR_INVALID_GRID,
+	GET_TABLE_ENTRY(gte, NULL, CUDBG_ERROR_INVALID_GRID,
 			"grid%llu_dev%u", gridId, dev);
 
 	gridDim->x = gte->gridDimX;
@@ -646,7 +653,7 @@ DEF_API_CALL(readGridId)(uint32_t dev, uint32_t sm, uint32_t wp,
 	if ((rc = getGridId(dev, sm, wp, &gridId)) != CUDBG_SUCCESS)
 		return rc;
 
-	GET_TABLE_ENTRY(gte, CUDBG_ERROR_INVALID_GRID,
+	GET_TABLE_ENTRY(gte, NULL, CUDBG_ERROR_INVALID_GRID,
 			"grid%llu_dev%u", gridId, dev);
 
 	*gridId64 = gte->gridId64;
@@ -658,6 +665,7 @@ DEF_API_CALL(getGridInfo)(uint32_t devId, uint64_t gridId, CUDBGGridInfo *info)
 {
 	CudbgGridTableEntry *gte;
 	CudbgContextTableEntry *cte;
+	size_t gteSize;
 
 	TRACE_FUNC("devId=%u gridId=%llu info=%p", devId, gridId, info);
 
@@ -666,10 +674,10 @@ DEF_API_CALL(getGridInfo)(uint32_t devId, uint64_t gridId, CUDBGGridInfo *info)
 	if (devId >= cuCoreGetNumDevices(curcc))
 		return CUDBG_ERROR_INVALID_DEVICE;
 
-	GET_TABLE_ENTRY(gte, CUDBG_ERROR_INVALID_GRID,
+	GET_TABLE_ENTRY(gte, &gteSize, CUDBG_ERROR_INVALID_GRID,
 			"grid%llu_dev%u", gridId, devId);
 
-	GET_TABLE_ENTRY(cte, CUDBG_ERROR_INVALID_CONTEXT,
+	GET_TABLE_ENTRY(cte, NULL, CUDBG_ERROR_INVALID_CONTEXT,
 			"ctx%llu_dev%u", gte->contextId, devId);
 
 	assert(gridId == gte->gridId64);
@@ -690,6 +698,12 @@ DEF_API_CALL(getGridInfo)(uint32_t devId, uint64_t gridId, CUDBGGridInfo *info)
 	info->blockDim.z = gte->blockDimZ;
 	info->parentGridId = gte->parentGridId64;
 	info->origin = gte->origin;
+	if (offsetof(CudbgGridTableEntry, clusterDimZ) < gteSize)
+	{
+		info->clusterDim.x = gte->clusterDimX;
+		info->clusterDim.y = gte->clusterDimY;
+		info->clusterDim.z = gte->clusterDimZ;
+	}
 
 	return CUDBG_SUCCESS;
 }
@@ -706,7 +720,7 @@ DEF_API_CALL(getGridStatus)(uint32_t devId, uint64_t gridId,
 	if (devId >= cuCoreGetNumDevices(curcc))
 		return CUDBG_ERROR_INVALID_DEVICE;
 
-	GET_TABLE_ENTRY(gte, CUDBG_ERROR_INVALID_GRID,
+	GET_TABLE_ENTRY(gte, NULL, CUDBG_ERROR_INVALID_GRID,
 			"grid%llu_dev%u", gridId, devId);
 
 	*status = gte->gridStatus;
@@ -745,7 +759,7 @@ DEF_API_CALL(readRegisterRange)(uint32_t devId, uint32_t sm, uint32_t wp,
 	if (index + registers_size > max_registers)
 		return CUDBG_ERROR_INVALID_ARGS;
 
-	GET_TABLE_ENTRY(scn, CUDBG_ERROR_INVALID_ARGS,
+	GET_TABLE_ENTRY(scn, NULL, CUDBG_ERROR_INVALID_ARGS,
 			"regs_dev%u_sm%u_wp%u_ln%u", devId, sm, wp, ln);
 
 	if (cuCoreReadSectionData(curcc->e, scn, &data) != 0)
@@ -794,7 +808,7 @@ DEF_API_CALL(readBrokenWarps)(uint32_t devId, uint32_t sm,
 
 	*brokenWarpsMask = 0;
 	for (wp = 0; wp < warpsPerSM; ++wp) {
-		wte = cuCoreGetMapEntry(&curcc->tableEntriesMap,
+		wte = cuCoreGetMapEntry(&curcc->tableEntriesMap, NULL,
 					"wp%u_sm%u_dev%u", wp, sm, devId);
 		if (wte && wte->isWarpBroken)
 			*brokenWarpsMask |= 1ULL << wp;
@@ -820,7 +834,7 @@ DEF_API_CALL(readValidWarps)(uint32_t devId, uint32_t sm,
 
 	*validWarpsMask = 0;
 	for (wp = 0; wp < warpsPerSM; ++wp) {
-		if (cuCoreGetMapEntry(&curcc->tableEntriesMap,
+		if (cuCoreGetMapEntry(&curcc->tableEntriesMap, NULL,
 				      "wp%u_sm%u_dev%u", wp, sm, devId))
 			*validWarpsMask |= 1ULL << wp;
 	}
@@ -838,7 +852,7 @@ DEF_API_CALL(readValidLanes)(uint32_t dev, uint32_t sm, uint32_t wp,
 
 	VERIFY_ARG(validLanesMask);
 
-	GET_TABLE_ENTRY(wte, CUDBG_ERROR_INVALID_WARP,
+	GET_TABLE_ENTRY(wte, NULL, CUDBG_ERROR_INVALID_WARP,
 			"wp%u_sm%u_dev%u", wp, sm, dev);
 
 	*validLanesMask = wte->validLanesMask;
@@ -856,7 +870,7 @@ DEF_API_CALL(readActiveLanes)(uint32_t dev, uint32_t sm, uint32_t wp,
 
 	VERIFY_ARG(activeLanesMask);
 
-	GET_TABLE_ENTRY(wte, CUDBG_ERROR_INVALID_WARP,
+	GET_TABLE_ENTRY(wte, NULL, CUDBG_ERROR_INVALID_WARP,
 			"wp%u_sm%u_dev%u", wp, sm, dev);
 
 	*activeLanesMask = wte->activeLanesMask;
@@ -873,7 +887,7 @@ DEF_API_CALL(getSmType)(uint32_t devId, char *buf, uint32_t sz)
 
 	VERIFY_ARG(buf);
 
-	GET_TABLE_ENTRY(dte, CUDBG_ERROR_INVALID_DEVICE, "dev%u", devId);
+	GET_TABLE_ENTRY(dte, NULL, CUDBG_ERROR_INVALID_DEVICE, "dev%u", devId);
 
 	smType = cuCoreGetStrTabByIndex(curcc, dte->smType);
 	strncpy(buf, smType, sz);
@@ -890,7 +904,7 @@ DEF_API_CALL(getDeviceName)(uint32_t devId, char *buf, uint32_t sz)
 
 	VERIFY_ARG(buf);
 
-	GET_TABLE_ENTRY(dte, CUDBG_ERROR_INVALID_DEVICE, "dev%u", devId);
+	GET_TABLE_ENTRY(dte, NULL, CUDBG_ERROR_INVALID_DEVICE, "dev%u", devId);
 
 	devName = cuCoreGetStrTabByIndex(curcc, dte->devName);
 	strncpy(buf, devName, sz);
@@ -909,7 +923,7 @@ DEF_API_CALL(getDevicePCIBusInfo)(uint32_t devId, uint32_t *pciBusId,
 	VERIFY_ARG(pciBusId);
 	VERIFY_ARG(pciDevId);
 
-	GET_TABLE_ENTRY(dte, CUDBG_ERROR_INVALID_DEVICE, "dev%u", devId);
+	GET_TABLE_ENTRY(dte, NULL, CUDBG_ERROR_INVALID_DEVICE, "dev%u", devId);
 
 	*pciDevId = dte->pciDevId;
 	*pciBusId = dte->pciBusId;
@@ -926,7 +940,7 @@ DEF_API_CALL(getDeviceType)(uint32_t devId, char *buf, uint32_t sz)
 
 	VERIFY_ARG(buf);
 
-	GET_TABLE_ENTRY(dte, CUDBG_ERROR_INVALID_DEVICE, "dev%u", devId);
+	GET_TABLE_ENTRY(dte, NULL, CUDBG_ERROR_INVALID_DEVICE, "dev%u", devId);
 
 	devType = cuCoreGetStrTabByIndex(curcc, dte->devType);
 	strncpy(buf, devType, sz);
@@ -1017,7 +1031,7 @@ DEF_API_CALL(readErrorPC)(uint32_t devId, uint32_t sm, uint32_t wp,
 	VERIFY_ARG(errorPC);
 	VERIFY_ARG(errorPCValid);
 
-	GET_TABLE_ENTRY(wte, CUDBG_ERROR_INVALID_WARP,
+	GET_TABLE_ENTRY(wte, NULL, CUDBG_ERROR_INVALID_WARP,
 			"wp%u_sm%u_dev%u", wp, sm, devId);
 
 	*errorPC = wte->errorPC;
@@ -1035,7 +1049,7 @@ DEF_API_CALL(readPC)(uint32_t devId, uint32_t sm, uint32_t wp, uint32_t ln,
 
 	VERIFY_ARG(pc);
 
-	GET_TABLE_ENTRY(tte, CUDBG_ERROR_INVALID_LANE,
+	GET_TABLE_ENTRY(tte, NULL, CUDBG_ERROR_INVALID_LANE,
 			"ln%u_wp%u_sm%u_dev%u", ln, wp, sm, devId);
 
 	*pc = tte->physPC;
@@ -1055,7 +1069,7 @@ DEF_API_CALL(getElfImageByHandle)(uint32_t devId, uint64_t handle,
 
 	VERIFY_ARG(elfImage);
 
-	GET_TABLE_ENTRY(scn, CUDBG_ERROR_INVALID_ARGS,
+	GET_TABLE_ENTRY(scn, NULL, CUDBG_ERROR_INVALID_ARGS,
 			"%celf_handle%llx",
 			type ? 'r':'u', handle);
 
@@ -1080,7 +1094,7 @@ DEF_API_CALL(readLaneException)(uint32_t dev, uint32_t sm, uint32_t wp,
 
 	VERIFY_ARG(exception);
 
-	GET_TABLE_ENTRY(tte, CUDBG_ERROR_INVALID_LANE,
+	GET_TABLE_ENTRY(tte, NULL, CUDBG_ERROR_INVALID_LANE,
 			"ln%u_wp%u_sm%u_dev%u", ln, wp, sm, dev);
 
 	*exception = (CUDBGException_t)tte->exception;
@@ -1098,7 +1112,7 @@ DEF_API_CALL(readLaneStatus)(uint32_t devId, uint32_t sm, uint32_t wp,
 
 	VERIFY_ARG(error);
 
-	GET_TABLE_ENTRY(tte, CUDBG_ERROR_INVALID_LANE,
+	GET_TABLE_ENTRY(tte, NULL, CUDBG_ERROR_INVALID_LANE,
 			"ln%u_wp%u_sm%u_dev%u", ln, wp, sm, devId);
 
 	*error = tte->exception != CUDBG_EXCEPTION_UNKNOWN;
@@ -1116,7 +1130,7 @@ DEF_API_CALL(readSyscallCallDepth)(uint32_t dev, uint32_t sm, uint32_t wp,
 
 	VERIFY_ARG(depth);
 
-	GET_TABLE_ENTRY(tte, CUDBG_ERROR_INVALID_LANE,
+	GET_TABLE_ENTRY(tte, NULL, CUDBG_ERROR_INVALID_LANE,
 			"ln%u_wp%u_sm%u_dev%u", ln, wp, sm, dev);
 
 	*depth = tte->syscallCallDepth;
@@ -1134,7 +1148,7 @@ DEF_API_CALL(readCallDepth)(uint32_t dev, uint32_t sm, uint32_t wp,
 
 	VERIFY_ARG(depth);
 
-	GET_TABLE_ENTRY(tte, CUDBG_ERROR_INVALID_LANE,
+	GET_TABLE_ENTRY(tte, NULL, CUDBG_ERROR_INVALID_LANE,
 			"ln%u_wp%u_sm%u_dev%u", ln, wp, sm, dev);
 
 	*depth = tte->callDepth;
@@ -1152,7 +1166,7 @@ DEF_API_CALL(readReturnAddress)(uint32_t dev, uint32_t sm, uint32_t wp,
 
 	VERIFY_ARG(ra);
 
-	GET_TABLE_ENTRY(bte, CUDBG_ERROR_INVALID_CALL_LEVEL,
+	GET_TABLE_ENTRY(bte, NULL, CUDBG_ERROR_INVALID_CALL_LEVEL,
 			"bt%u_ln%u_wp%u_sm%u_dev%u", level, ln, wp, sm, dev);
 
 	*ra = bte->returnAddress;
@@ -1171,7 +1185,7 @@ DEF_API_CALL(readVirtualReturnAddress)(uint32_t dev, uint32_t sm, uint32_t wp,
 
 	VERIFY_ARG(ra);
 
-	GET_TABLE_ENTRY(bte, CUDBG_ERROR_INVALID_CALL_LEVEL,
+	GET_TABLE_ENTRY(bte, NULL, CUDBG_ERROR_INVALID_CALL_LEVEL,
 			"bt%u_ln%u_wp%u_sm%u_dev%u", level, ln, wp, sm, dev);
 
 	*ra = bte->virtualReturnAddress;
@@ -1230,7 +1244,7 @@ DEF_API_CALL(getNumPredicates)(uint32_t dev, uint32_t *numPredicates)
 
 	VERIFY_ARG(numPredicates);
 
-	GET_TABLE_ENTRY(dte, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
+	GET_TABLE_ENTRY(dte, NULL, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
 
 	*numPredicates = dte->numPredicatesPrLane;
 
@@ -1263,7 +1277,7 @@ DEF_API_CALL(readPredicates)(uint32_t dev, uint32_t sm, uint32_t wp,
 	if (predicates_size > num_predicates)
 		return CUDBG_ERROR_INVALID_ARGS;
 
-	GET_TABLE_ENTRY(scn, CUDBG_ERROR_INVALID_ARGS,
+	GET_TABLE_ENTRY(scn, NULL, CUDBG_ERROR_INVALID_ARGS,
 			"pred_dev%u_sm%u_wp%u_ln%u", dev, sm, wp, ln);
 
 	if (cuCoreReadSectionData(curcc->e, scn, &data) != 0)
@@ -1289,7 +1303,7 @@ DEF_API_CALL(readCCRegister)(uint32_t dev, uint32_t sm, uint32_t wp,
 
 	VERIFY_ARG(val);
 
-	GET_TABLE_ENTRY(tte, CUDBG_ERROR_INVALID_LANE,
+	GET_TABLE_ENTRY(tte, NULL, CUDBG_ERROR_INVALID_LANE,
 			"ln%u_wp%u_sm%u_dev%u", ln, wp, sm, dev);
 
 	*val = tte->ccRegister;
@@ -1315,7 +1329,7 @@ DEF_API_CALL(getGridAttributes)(uint32_t dev, uint32_t sm, uint32_t wp,
 	if ((rc = getGridId(dev, sm, wp, &gridId)) != CUDBG_SUCCESS)
 		return rc;
 
-	GET_TABLE_ENTRY(gte, CUDBG_ERROR_INVALID_GRID,
+	GET_TABLE_ENTRY(gte, NULL, CUDBG_ERROR_INVALID_GRID,
 			"grid%llu_dev%u", gridId, dev);
 
 	for (pairId = 0; pairId < numPairs; ++pairId) {
@@ -1397,18 +1411,24 @@ DEF_API_CALL(disassemble)(uint32_t dev, uint64_t addr, uint32_t *instSize,
 
 	TRACE_FUNC("dev=%u addr=0x%llx instSize=%p buf=%p sz=%u", dev, addr, instSize, buf, sz);
 
-	VERIFY_ARG(instSize && (!sz || buf));
+	VERIFY_ARG(instSize);
+
+	if (buf)
+		*buf = '\0';
 
 	/* This assumes that the GPU architecture has a uniform instruction
 	 * size, which is true on all GPU architectures except FERMI.
 	 * Since cuda-gdb no longer supports FERMI as of 9.0 toolkit, this
 	 * assumption is valid. */
-	GET_TABLE_ENTRY(dte, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
+	GET_TABLE_ENTRY(dte, NULL, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
 	*instSize = dte->instructionSize;
 
 	if (!sz)
 		return CUDBG_SUCCESS;
 
+	/* Verify a buf argument was passed after we verify sz != 0 */
+	VERIFY_ARG(buf);
+	
 	/* Disassembling from device memory with a coredump is invalid */
 	return CUDBG_ERROR_INVALID_DEVICE;
 }
@@ -1416,14 +1436,18 @@ DEF_API_CALL(disassemble)(uint32_t dev, uint64_t addr, uint32_t *instSize,
 DEF_API_CALL(getNumUniformRegisters)(uint32_t dev, uint32_t *numRegs)
 {
 	CudbgDeviceTableEntry *dte;
+	size_t dteSize;
 
 	TRACE_FUNC("dev=%u numRegs=%p", dev, numRegs);
 
 	VERIFY_ARG(numRegs);
 
-	GET_TABLE_ENTRY(dte, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
+	GET_TABLE_ENTRY(dte, &dteSize, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
 
-	*numRegs = dte->numUniformRegsPrWarp;
+	if (offsetof(CudbgDeviceTableEntry, numUniformRegsPrWarp) < dteSize)
+		*numRegs = dte->numUniformRegsPrWarp;
+	else
+		*numRegs = 0;
 
 	return CUDBG_SUCCESS;
 }
@@ -1452,7 +1476,7 @@ DEF_API_CALL(readUniformRegisterRange)(uint32_t devId, uint32_t sm, uint32_t wp,
 	if (devId >= cuCoreGetNumDevices(curcc))
 		return CUDBG_ERROR_INVALID_DEVICE;
 
-	GET_TABLE_ENTRY(scn, CUDBG_ERROR_MISSING_DATA,
+	GET_TABLE_ENTRY(scn, NULL, CUDBG_ERROR_MISSING_DATA,
 			"uregs_dev%u_sm%u_wp%u", devId, sm, wp);
 
 	rc = API_CALL(getNumUniformRegisters)(devId, &max_registers);
@@ -1484,14 +1508,18 @@ DEF_API_CALL(readUniformRegisterRange)(uint32_t devId, uint32_t sm, uint32_t wp,
 DEF_API_CALL(getNumUniformPredicates)(uint32_t dev, uint32_t *numPredicates)
 {
 	CudbgDeviceTableEntry *dte;
+	size_t dteSize;
 
 	TRACE_FUNC("dev=%u numPredicates=%p", dev, numPredicates);
 
 	VERIFY_ARG(numPredicates);
 
-	GET_TABLE_ENTRY(dte, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
+	GET_TABLE_ENTRY(dte, &dteSize, CUDBG_ERROR_INVALID_DEVICE, "dev%u", dev);
 
-	*numPredicates = dte->numUniformPredicatesPrWarp;
+	if (offsetof(CudbgDeviceTableEntry, numUniformPredicatesPrWarp) < dteSize)
+		*numPredicates = dte->numUniformPredicatesPrWarp;
+	else
+		*numPredicates = 0;
 
 	return CUDBG_SUCCESS;
 }
@@ -1518,7 +1546,7 @@ DEF_API_CALL(readUniformPredicates)(uint32_t devId, uint32_t sm, uint32_t wp,
 	if (devId >= cuCoreGetNumDevices(curcc))
 		return CUDBG_ERROR_INVALID_DEVICE;
 
-	GET_TABLE_ENTRY(scn, CUDBG_ERROR_MISSING_DATA,
+	GET_TABLE_ENTRY(scn, NULL, CUDBG_ERROR_MISSING_DATA,
 			"upred_dev%u_sm%u_wp%u", devId, sm, wp);
 
 	rc = API_CALL(getNumUniformPredicates)(devId, &max_predicates);
@@ -1543,6 +1571,57 @@ DEF_API_CALL(readUniformPredicates)(uint32_t devId, uint32_t sm, uint32_t wp,
 	return rc;
 }
 
+DEF_API_CALL(getClusterDim)(uint32_t devId, uint64_t gridId, CuDim3 *clusterDim)
+{
+	CudbgGridTableEntry *gte;
+	size_t gteSize;
+
+	TRACE_FUNC("devId=%u gridId=%llu clusterDim=%p", devId, gridId, clusterDim);
+
+	VERIFY_ARG(clusterDim);
+
+	if (devId >= cuCoreGetNumDevices(curcc))
+		return CUDBG_ERROR_INVALID_DEVICE;
+
+	GET_TABLE_ENTRY(gte, &gteSize, CUDBG_ERROR_INVALID_GRID,
+			"grid%llu_dev%u", gridId, devId);
+
+	assert(gridId == gte->gridId64);
+	memset(clusterDim, 0, sizeof(*clusterDim));
+	if (offsetof(CudbgGridTableEntry, clusterDimZ) < gteSize)
+	{
+		clusterDim->x = gte->clusterDimX;
+		clusterDim->y = gte->clusterDimY;
+		clusterDim->z = gte->clusterDimZ;
+	}
+
+	return CUDBG_SUCCESS;
+}
+
+DEF_API_CALL(readClusterIdx)(uint32_t dev, uint32_t sm, uint32_t wp,
+			     CuDim3 *clusterIdx)
+{
+	CudbgCTATableEntry *ctate;
+	size_t ctateSize;
+
+	TRACE_FUNC("dev=%u sm=%u wp=%u clusterIdx=%p", dev, sm, wp, clusterIdx);
+
+	VERIFY_ARG(clusterIdx);
+
+	GET_TABLE_ENTRY(ctate, &ctateSize, CUDBG_ERROR_UNKNOWN,
+			"wp%u_sm%u_dev%u_cta", wp, sm, dev);
+
+	memset(clusterIdx, 0, sizeof(*clusterIdx));
+	if (offsetof(CudbgCTATableEntry, clusterIdxZ) < ctateSize)
+	{
+		clusterIdx->x = ctate->clusterIdxX;
+		clusterIdx->y = ctate->clusterIdxY;
+		clusterIdx->z = ctate->clusterIdxZ;
+	}
+
+	return CUDBG_SUCCESS;
+}
+
 static const struct CUDBGAPI_st cudbgCoreApi = {
     /* Initialization */
     API_CALL(doNothing),
@@ -1663,7 +1742,7 @@ static const struct CUDBGAPI_st cudbgCoreApi = {
     /* 5.5 Extensions */
     API_CALL(notSupported),
     API_CALL(notSupported),
-    API_CALL(getGridInfo),
+    API_CALL(notSupported),
     API_CALL(readGridId),
     API_CALL(getGridStatus),
     API_CALL(notSupported),
@@ -1676,7 +1755,7 @@ static const struct CUDBGAPI_st cudbgCoreApi = {
     API_CALL(getNextEvent),
     API_CALL(getElfImageByHandle),
     API_CALL(notSupported),
-    API_CALL(readWarpState),
+    API_CALL(notSupported),
     API_CALL(readRegisterRange),
     API_CALL(readGenericMemory),
     API_CALL(notSupported),
@@ -1706,11 +1785,20 @@ static const struct CUDBGAPI_st cudbgCoreApi = {
     API_CALL(getNumUniformPredicates),
     API_CALL(readUniformPredicates),
     API_CALL(notSupported),
+
+    /* 11.8 Extensions */
+    API_CALL(notSupported),
+
+    /* 12.0 Extensions */
+    API_CALL(getGridInfo),
+    API_CALL(getClusterDim),
+    API_CALL(readWarpState),
+    API_CALL(readClusterIdx),
 };
 
 CUDBGAPI cuCoreGetApi(CudaCore *cc)
 {
-	curcc = cc;
+  curcc = cc;
 
-	return &cudbgCoreApi;
+  return &cudbgCoreApi;
 }
